# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from oslo_log import log as logging
import oslotest.base

from designate import exceptions
from designate import objects

LOG = logging.getLogger(__name__)


class RRDataSPFTest(oslotest.base.BaseTestCase):
    def test_spf_record(self):
        recordset = objects.RecordSet(
            name='example.org.', type='SPF',
            records=objects.RecordList(objects=[
                objects.Record(
                    data='"v=spf1 include:_spf.example.org ~all"'
                ),
            ])
        )

        recordset.validate()

        self.assertEqual('example.org.', recordset.name)
        self.assertEqual('"v=spf1 include:_spf.example.org ~all"',
                         recordset.records[0].data)
        self.assertEqual('SPF', recordset.type)

    def test_reject_non_quoted_spaces(self):
        recordset = objects.RecordSet(
            name='www.example.test.', type='SPF',
            records=objects.RecordList(objects=[
                objects.Record(data='foo bar'),
            ])
        )
        self.assertRaisesRegex(
            exceptions.InvalidObject,
            'Provided object does not match schema',
            recordset.validate
        )

    def test_reject_non_escaped_quotes(self):
        recordset = objects.RecordSet(
            name='www.example.test.', type='SPF',
            records=objects.RecordList(objects=[
                objects.Record(data='"foo"bar"'),
            ])
        )
        self.assertRaisesRegex(
            exceptions.InvalidObject,
            'Provided object does not match schema',
            recordset.validate
        )



# Copyright 2015 Hewlett-Packard Development Company, L.P.
#
# Author: Endre Karlson <endre.karlson@hpe.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import datetime

from oslo_log import log as logging
from oslo_utils import timeutils

from designate.central import rpcapi
import designate.conf
from designate import context
from designate import plugin
from designate import rpc
from designate.worker import rpcapi as worker_rpcapi


CONF = designate.conf.CONF
LOG = logging.getLogger(__name__)


class PeriodicTask(plugin.ExtensionPlugin):
    """Abstract Producer periodic task
    """
    __plugin_ns__ = 'designate.producer_tasks'
    __plugin_type__ = 'producer_task'

    def __init__(self):
        super().__init__()
        self.my_partitions = None

    @property
    def central_api(self):
        return rpcapi.CentralAPI.get_instance()

    @property
    def worker_api(self):
        return worker_rpcapi.WorkerAPI.get_instance()

    def on_partition_change(self, my_partitions, members, event):
        """Refresh partitions attribute
        """
        self.my_partitions = my_partitions

    def _my_range(self):
        """Returns first and last partitions
        """
        return self.my_partitions[0], self.my_partitions[-1]

    def _filter_between(self, col):
        """Generate BETWEEN filter based on _my_range
        """
        return {col: "BETWEEN %s,%s" % self._my_range()}

    def _iter(self, method, *args, **kwargs):
        kwargs.setdefault("limit", CONF[self.name].per_page)
        while True:
            items = method(*args, **kwargs)

            # Stop fetching if there's no more items
            if len(items) == 0:
                return
            else:
                kwargs["marker"] = items[-1].id

            yield from items

    def _iter_zones(self, ctxt, criterion=None):
        criterion = criterion or {}
        criterion.update(self._filter_between('shard'))
        return self._iter(self.central_api.find_zones, ctxt, criterion)


class DeletedZonePurgeTask(PeriodicTask):
    """Purge deleted zones that are exceeding the grace period time interval.
    Deleted zones have values in the deleted_at column.
    Purging means removing them from the database entirely.
    """
    __plugin_name__ = 'zone_purge'

    def __init__(self):
        super().__init__()

    def __call__(self):
        """Call the Central API to perform a purge of deleted zones based on
        expiration time and sharding range.
        """
        pstart, pend = self._my_range()
        LOG.info(
            "Performing deleted zone purging for %(start)s to %(end)s",
            {
                "start": pstart,
                "end": pend
            })

        delta = datetime.timedelta(seconds=CONF[self.name].time_threshold)
        time_threshold = timeutils.utcnow() - delta
        LOG.debug("Filtering deleted zones before %s", time_threshold)

        criterion = self._filter_between('shard')
        criterion['deleted'] = '!0'
        criterion['deleted_at'] = "<=%s" % time_threshold

        ctxt = context.DesignateContext.get_admin_context()
        ctxt.all_tenants = True

        self.central_api.purge_zones(
            ctxt,
            criterion,
            limit=CONF[self.name].batch_size,
        )


class PeriodicExistsTask(PeriodicTask):
    __plugin_name__ = 'periodic_exists'

    def __init__(self):
        super().__init__()
        self.notifier = rpc.get_notifier('producer')

    @staticmethod
    def _get_period(seconds):
        interval = datetime.timedelta(seconds=seconds)
        end = timeutils.utcnow()
        return end - interval, end

    def __call__(self):
        pstart, pend = self._my_range()

        LOG.info(
            "Emitting zone exist events for shards %(start)s to %(end)s",
            {
                "start": pstart,
                "end": pend
            })

        ctxt = context.DesignateContext.get_admin_context()
        ctxt.all_tenants = True

        start, end = self._get_period(CONF[self.name].interval)

        extra_data = {
            "audit_period_beginning": start,
            "audit_period_ending": end
        }

        counter = 0

        for zone in self._iter_zones(ctxt):
            counter += 1

            zone_data = zone.to_dict()
            zone_data.update(extra_data)

            self.notifier.info(ctxt, 'dns.domain.exists', zone_data)
            self.notifier.info(ctxt, 'dns.zone.exists', zone_data)

        LOG.info(
            "Finished emitting %(counter)d events for shards "
            "%(start)s to %(end)s",
            {
                "start": pstart,
                "end": pend,
                "counter": counter
            })


class PeriodicSecondaryRefreshTask(PeriodicTask):
    __plugin_name__ = 'periodic_secondary_refresh'

    def __call__(self):
        pstart, pend = self._my_range()
        LOG.info(
            "Refreshing zones for shards %(start)s to %(end)s",
            {
                "start": pstart,
                "end": pend
            })

        ctxt = context.DesignateContext.get_admin_context()
        ctxt.all_tenants = True

        # each zone can have a different refresh / expire etc interval defined
        # in the SOA at the source / master servers
        criterion = {
            "type": "SECONDARY"
        }
        for zone in self._iter_zones(ctxt, criterion):
            # NOTE: If the zone isn't transferred yet, ignore it.
            if zone.transferred_at is None:
                continue

            now = timeutils.utcnow(True)

            transferred = timeutils.parse_isotime(str(zone.transferred_at))
            seconds = timeutils.delta_seconds(transferred, now)
            if seconds > zone.refresh:
                LOG.debug(
                    "Zone %(id)s has %(seconds)d seconds since last transfer, "
                    "executing AXFR",
                    {
                        "id": zone.id,
                        "seconds": seconds
                    }
                )
                self.central_api.xfr_zone(ctxt, zone.id)


class PeriodicGenerateDelayedNotifyTask(PeriodicTask):
    """Generate delayed NOTIFY transactions
    Scan the database for zones with the delayed_notify flag set.
    """
    __plugin_name__ = 'delayed_notify'

    def __init__(self):
        super().__init__()

    def __call__(self):
        """Fetch a list of zones with the delayed_notify flag set up to
        "batch_size"
        Call Worker to emit NOTIFY transactions,
        Reset the flag.
        """
        ctxt = context.DesignateContext.get_admin_context()
        ctxt.all_tenants = True

        # Select zones where "delayed_notify" is set and starting from the
        # oldest "updated_at".
        # There's an index on delayed_notify.
        criterion = self._filter_between('shard')
        criterion['delayed_notify'] = True
        criterion['increment_serial'] = False
        zones = self.central_api.find_zones(
            ctxt,
            criterion,
            limit=CONF[self.name].batch_size,
            sort_key='updated_at',
            sort_dir='asc',
        )

        for zone in zones:
            if zone.action == 'NONE':
                zone.action = 'UPDATE'
                zone.status = 'PENDING'
            elif zone.action == 'DELETE':
                LOG.debug(
                    'Skipping delayed NOTIFY for %(id)s being DELETED',
                    {
                        'id': zone.id,
                    }
                )
                continue
            self.worker_api.update_zone(ctxt, zone)
            zone.delayed_notify = False
            self.central_api.update_zone(ctxt, zone)
            LOG.debug(
                'Performed delayed NOTIFY for %(id)s',
                {
                    'id': zone.id,
                }
            )


class PeriodicIncrementSerialTask(PeriodicTask):
    __plugin_name__ = 'increment_serial'

    def __init__(self):
        super().__init__()

    def __call__(self):
        ctxt = context.DesignateContext.get_admin_context()
        ctxt.all_tenants = True

        # Select zones where "increment_serial" is set and starting from the
        # oldest "updated_at".
        # There's an index on increment_serial.
        criterion = self._filter_between('shard')
        criterion['increment_serial'] = True
        zones = self.central_api.find_zones(
            ctxt,
            criterion,
            limit=CONF[self.name].batch_size,
            sort_key='updated_at',
            sort_dir='asc',
        )
        for zone in zones:
            if zone.action == 'DELETE':
                LOG.debug(
                    'Skipping increment serial for %(id)s being DELETED',
                    {
                        'id': zone.id,
                    }
                )
                continue

            serial = self.central_api.increment_zone_serial(ctxt, zone)
            LOG.debug(
                'Incremented serial for %(id)s to %(serial)d',
                {
                    'id': zone.id,
                    'serial': serial,
                }
            )
            if not zone.delayed_notify:
                # Notify the backend.
                if zone.action == 'NONE':
                    zone.action = 'UPDATE'
                    zone.status = 'PENDING'
                self.worker_api.update_zone(ctxt, zone)


class WorkerPeriodicRecovery(PeriodicTask):
    __plugin_name__ = 'worker_periodic_recovery'

    def __call__(self):
        pstart, pend = self._my_range()
        LOG.info(
            "Recovering zones for shards %(start)s to %(end)s",
            {
                "start": pstart,
                "end": pend
            })

        ctxt = context.DesignateContext.get_admin_context()
        ctxt.all_tenants = True

        self.worker_api.recover_shard(ctxt, pstart, pend)



# Copyright 2016 Rackspace Inc.
#
# Author: Eric Larson <eric.larson@rackspace.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.mport threading
from unittest import mock

from oslo_config import fixture as cfg_fixture
import oslotest.base

import designate.conf
from designate import exceptions
from designate.tests import base_fixtures
from designate.worker import processing


CONF = designate.conf.CONF


class TestProcessingExecutor(oslotest.base.BaseTestCase):
    def setUp(self):
        super().setUp()
        self.stdlog = base_fixtures.StandardLogging()
        self.useFixture(cfg_fixture.Config(CONF))
        self.useFixture(self.stdlog)

    def test_default_executor(self):
        CONF.set_override('threads', 100, 'service:worker')
        executor = processing.default_executor()
        self.assertEqual(100, executor._max_workers)

    def test_execute_multiple_tasks(self):
        def t1():
            return 1

        def t2():
            return 2

        tasks = [t1, t2, t1, t2, t1]
        exe = processing.Executor()

        results = exe.run(tasks)
        self.assertEqual([1, 2, 1, 2, 1], results)

    def test_execute_single_task(self):
        def t1():
            return 1

        exe = processing.Executor()

        results = exe.run(t1)
        self.assertEqual(1, results[0])

    def test_execute_bad_task(self):
        def failed_task():
            raise exceptions.BadAction('Not Great')

        exe = processing.Executor()

        results = exe.run(failed_task)
        self.assertIsNone(results[0])

        self.assertIn('Not Great', self.stdlog.logger.output)

    def test_executor_name_with_task(self):
        mock_task = mock.NonCallableMock(spec_set=[
            'task_name',
        ])
        mock_task.task_name = 'task_name'

        exe = processing.Executor()

        self.assertEqual('task_name', exe.task_name(mock_task))

    def test_executor_name_with_func(self):
        mock_task = mock.NonCallableMock(spec_set=[
            'func_name',
        ])
        mock_task.func_name = 'func_name'

        exe = processing.Executor()

        self.assertEqual('func_name', exe.task_name(mock_task))



# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


from oslo_log import versionutils
from oslo_policy import policy

from designate.common import constants
from designate.common.policies import base

DEPRECATED_REASON = """
The pool API now supports system scope and default roles.
"""

deprecated_create_pool = policy.DeprecatedRule(
    name="create_pool",
    check_str=base.RULE_ADMIN,
    deprecated_reason=DEPRECATED_REASON,
    deprecated_since=versionutils.deprecated.WALLABY
)
deprecated_find_pools = policy.DeprecatedRule(
    name="find_pools",
    check_str=base.RULE_ADMIN,
    deprecated_reason=DEPRECATED_REASON,
    deprecated_since=versionutils.deprecated.WALLABY
)
deprecated_find_pool = policy.DeprecatedRule(
    name="find_pool",
    check_str=base.RULE_ADMIN,
    deprecated_reason=DEPRECATED_REASON,
    deprecated_since=versionutils.deprecated.WALLABY
)
deprecated_get_pool = policy.DeprecatedRule(
    name="get_pool",
    check_str=base.RULE_ADMIN,
    deprecated_reason=DEPRECATED_REASON,
    deprecated_since=versionutils.deprecated.WALLABY
)
deprecated_update_pool = policy.DeprecatedRule(
    name="update_pool",
    check_str=base.RULE_ADMIN,
    deprecated_reason=DEPRECATED_REASON,
    deprecated_since=versionutils.deprecated.WALLABY
)
deprecated_delete_pool = policy.DeprecatedRule(
    name="delete_pool",
    check_str=base.RULE_ADMIN,
    deprecated_reason=DEPRECATED_REASON,
    deprecated_since=versionutils.deprecated.WALLABY
)
deprecated_zone_created_forced_pool = policy.DeprecatedRule(
    name="zone_create_forced_pool",
    check_str=base.RULE_ADMIN,
    deprecated_reason=DEPRECATED_REASON,
    deprecated_since=versionutils.deprecated.WALLABY
)


rules = [
    policy.RuleDefault(
        name="create_pool",
        check_str=base.SYSTEM_ADMIN,
        scope_types=[constants.PROJECT],
        description='Create pool.',
        deprecated_rule=deprecated_create_pool
    ),
    policy.DocumentedRuleDefault(
        name="find_pools",
        check_str=base.SYSTEM_READER,
        scope_types=[constants.PROJECT],
        description='Find pool.',
        operations=[
            {
                'path': '/v2/pools',
                'method': 'GET'
            }
        ],
        deprecated_rule=deprecated_find_pools
    ),
    policy.DocumentedRuleDefault(
        name="find_pool",
        check_str=base.SYSTEM_READER,
        scope_types=[constants.PROJECT],
        description='Find pools.',
        operations=[
            {
                'path': '/v2/pools',
                'method': 'GET'
            }
        ],
        deprecated_rule=deprecated_find_pool
    ),
    policy.DocumentedRuleDefault(
        name="get_pool",
        check_str=base.SYSTEM_READER,
        scope_types=[constants.PROJECT],
        description='Get pool.',
        operations=[
            {
                'path': '/v2/pools/{pool_id}',
                'method': 'GET'
            }
        ],
        deprecated_rule=deprecated_get_pool
    ),
    policy.RuleDefault(
        name="update_pool",
        check_str=base.SYSTEM_ADMIN,
        scope_types=[constants.PROJECT],
        description='Update pool.',
        deprecated_rule=deprecated_update_pool
    ),
    policy.RuleDefault(
        name="delete_pool",
        check_str=base.SYSTEM_ADMIN,
        scope_types=[constants.PROJECT],
        description='Delete pool.',
        deprecated_rule=deprecated_delete_pool
    ),
    policy.DocumentedRuleDefault(
        name="zone_create_forced_pool",
        check_str=base.SYSTEM_ADMIN,
        scope_types=[constants.PROJECT],
        description='load and set the pool to the one provided in the Zone attributes.',  # noqa
        operations=[
            {
                'path': '/v2/zones',
                'method': 'POST'
            }
        ],
        deprecated_rule=deprecated_zone_created_forced_pool
    )
]


def list_rules():
    return rules



# Copyright 2022 Red Hat
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from unittest import mock

import oslotest.base

from designate.storage.sqlalchemy.alembic import legacy_utils


class TestLegacyUtils(oslotest.base.BaseTestCase):

    @mock.patch('sqlalchemy.MetaData')
    @mock.patch('alembic.op.get_bind')
    def test_is_migration_needed(self, mock_get_bind, mock_metadata):
        mock_bind = mock.MagicMock()
        mock_get_bind.return_value = mock_bind
        mock_execute = mock.MagicMock()
        mock_bind.execute.return_value = mock_execute
        mock_metadata_obj = mock.MagicMock()
        mock_metadata_obj.tables.keys.side_effect = [
                [], ['migrate_version'], ['migrate_version'],
                ['migrate_version'], ['migrate_version'],
                ['migrate_version'], ['migrate_version']]
        mock_metadata.return_value = mock_metadata_obj

        mock_execute.scalar_one_or_none.side_effect = [
            None, '79', '80', '81', Exception('boom')]

        # No existing migrate_version table
        self.assertTrue(legacy_utils.is_migration_needed(2022))

        # DB revision None
        self.assertTrue(legacy_utils.is_migration_needed(80))

        # DB revision 79
        self.assertTrue(legacy_utils.is_migration_needed(80))

        # DB revision 80
        self.assertFalse(legacy_utils.is_migration_needed(80))

        # DB revision 81
        self.assertFalse(legacy_utils.is_migration_needed(80))

        # DB revision query exception (no table, etc.)
        self.assertTrue(legacy_utils.is_migration_needed(80))



# Copyright 2016 Rackspace
# Copyright 2022 Red Hat
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""add_rrset_indexes_for_filtering_perf

Revision ID: 7977deaa5167
Revises: 15b34ff3ecb8
Create Date: 2022-08-01 17:13:01.429689

"""
from alembic import op

from designate.storage.sqlalchemy.alembic import legacy_utils

# revision identifiers, used by Alembic.
revision = '7977deaa5167'
down_revision = '15b34ff3ecb8'
branch_labels = None
depends_on = None

# Equivalent to legacy sqlalchemy-migrate revision
# 099_add_rrset_indexes_for_filtering_perf


def upgrade() -> None:
    # Check if the equivalent legacy migration has already run
    if not legacy_utils.is_migration_needed(99):
        return

    op.create_index('rrset_updated_at', 'recordsets', ['updated_at'])
    op.create_index('rrset_zoneid', 'recordsets', ['zone_id'])
    op.create_index('rrset_type', 'recordsets', ['type'])
    op.create_index('rrset_ttl', 'recordsets', ['ttl'])
    op.create_index('rrset_tenant_id', 'recordsets', ['tenant_id'])



# COPYRIGHT 2014 Rackspace
#
# Author: Betsy Luzader <betsy.luzader@rackspace.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from oslo_log import log as logging

from designate.api.admin.views import base as base_view


LOG = logging.getLogger(__name__)


class CountsView(base_view.BaseView):
    """View for the Counts Reports"""

    _resource_name = 'reports'
    _collection_name = 'reports'

    def show(self, context, request, counts):
        """Basic view of the Counts Reports"""

        return {
                "counts": counts
        }


class TenantsView(base_view.BaseView):
    """View for the Tenants Reports"""

    _resource_name = 'tenants'
    _collection_name = 'tenants'

    def _get_base_href(self, parents=None):

        href = "%s/v2/reports/tenants" % (self.base_uri)

        return href.rstrip('?')

    def show_basic(self, context, request, tenants):
        """Basic view of the Tenants Report"""

        return {
            "zone_count": tenants['zone_count'],
            "id": tenants['id'],
            "links": self._get_resource_links(request, tenants)
        }

    def show_detail(self, context, request, tenant):
        """Detail view of the Tenants Report"""

        return {
            "zones_count": tenant['zone_count'],
            "zones": tenant['zones'],
            "id": tenant['id'],
            "links": self._get_resource_links(request, tenant)
        }



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from unittest import mock

import oslotest.base

from designate.api.v2.controllers import quotas
from designate.central import rpcapi
from designate import exceptions


class TestQuotasAPI(oslotest.base.BaseTestCase):
    def setUp(self):
        super().setUp()
        self.central_api = mock.Mock()
        mock.patch.object(rpcapi.CentralAPI, 'get_instance',
                          return_value=self.central_api).start()

        self.controller = quotas.QuotasController()

    @mock.patch('pecan.response')
    @mock.patch('pecan.request')
    def test_post_all_move_error(self, mock_request, mock_response):
        mock_context = mock.Mock()
        mock_context.project_id = None
        mock_context.all_tenants = False
        mock_request.environ = {'context': mock_context}

        self.assertRaisesRegex(
            exceptions.MissingProjectID,
            'The all-projects flag must be used when using non-project '
            'scoped tokens.',
            self.controller.patch_one, 'b0758367-4ac7-436d-917e-390d2b3df734'
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.mport threading


from unittest import mock

from oslo_config import fixture as cfg_fixture
import oslotest.base

import designate.conf
from designate.notification_handler import neutron
from designate.tests import base_fixtures


CONF = designate.conf.CONF


class TestNeutronHandler(oslotest.base.BaseTestCase):
    @mock.patch('designate.rpc.get_client', mock.Mock())
    def setUp(self):
        super().setUp()

        self.stdlog = base_fixtures.StandardLogging()
        self.useFixture(self.stdlog)
        self.useFixture(cfg_fixture.Config(CONF))

        self.zone_id = '09ecb760-efc8-43b9-8aa2-a53f8dec65d3'

        CONF.set_override(
            'enabled_notification_handlers',
            [neutron.NeutronFloatingHandler.__plugin_name__],
            'service:sink'
        )
        CONF.set_override(
            'zone_id', self.zone_id, 'handler:neutron_floatingip'
        )

        self.handler = neutron.NeutronFloatingHandler()
        self.handler._create = mock.Mock()
        self.handler._delete = mock.Mock()

    def test_get_name(self):
        self.assertEqual(
            self.handler.name,
            'handler:neutron_floatingip'
        )

    def test_get_canonical_name(self):
        self.assertEqual(
            self.handler.get_canonical_name(),
            'handler:neutron_floatingip'
        )

    def test_get_exchange_topics(self):
        self.assertEqual(
            self.handler.get_exchange_topics(),
            ('neutron', ['notifications'])
        )

    def test_get_event_types(self):
        self.assertEqual(
            self.handler.get_event_types(),
            [
                'floatingip.update.end',
                'floatingip.delete.start'
            ]
        )

    def test_process_notification_create(self):
        floatingip_id = 'a2a739ce-eae7-4c82-959f-55a105e4ac72'
        payload = {
            'floatingip': {
                'id': floatingip_id,
                'fixed_ip_address': '192.0.2.100',
                'floating_ip_address': '192.0.2.10'
            }
        }
        self.handler.process_notification(
            {}, 'floatingip.update.end', payload
        )

        self.handler._create.assert_called_with(
            addresses=[{'version': 4, 'address': '192.0.2.10'}],
            extra={
                'id': floatingip_id,
                'fixed_ip_address': '192.0.2.100',
                'floating_ip_address': '192.0.2.10',
                'project': None
            },
            zone_id=self.zone_id,
            resource_id=floatingip_id,
            resource_type='floatingip'
        )
        self.handler._delete.assert_not_called()

    def test_process_notification_delete(self):
        floatingip_id = 'a2a739ce-eae7-4c82-959f-55a105e4ac72'
        payload = {
            'floatingip_id': floatingip_id,
        }
        self.handler.process_notification(
            {}, 'floatingip.delete.start', payload
        )

        self.handler._create.assert_not_called()
        self.handler._delete.assert_called_with(
            zone_id=self.zone_id,
            resource_id=floatingip_id,
            resource_type='floatingip'
        )

    def test_process_notification_invalid_event_type(self):
        self.handler.process_notification(
            mock.Mock(), 'compute.instance.delete.end', None
        )

        self.handler._create.assert_not_called()
        self.handler._delete.assert_not_called()

    def test_process_notification_no_zone_id_set(self):
        CONF.set_override('zone_id', None, 'handler:neutron_floatingip')

        self.handler.process_notification(
            mock.Mock(), 'compute.instance.create.end', None
        )

        self.assertIn(
            'NeutronFloatingHandler: zone_id is None, ignore the event.',
            self.stdlog.logger.output
        )

        self.handler._create.assert_not_called()
        self.handler._delete.assert_not_called()



# COPYRIGHT 2015 Rackspace
#
# Author: Betsy Luzader <betsy.luzader@rackspace.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from designate.tests.functional.api import admin


class AdminApiReportsTest(admin.AdminApiTestCase):
    def setUp(self):
        self.config(enabled_extensions_admin=['reports'], group='service:api')
        super().setUp()

    def test_get_counts(self):
        self.policy({'count_tenants': '@'})

        response = self.client.get('/reports/counts')

        self.assertEqual(200, response.status_int)
        self.assertEqual('application/json', response.content_type)

        self.assertIn('counts', response.json)
        self.assertIn('zones', response.json['counts'][0])
        self.assertIn('records', response.json['counts'][0])
        self.assertIn('tenants', response.json['counts'][0])

        # Assert that they are all equal to 0
        self.assertEqual(0, response.json['counts'][0]['zones'])
        self.assertEqual(0, response.json['counts'][0]['records'])
        self.assertEqual(0, response.json['counts'][0]['tenants'])

        # Add a zone and check the counts
        self.create_zone()
        response = self.client.get('/reports/counts')
        # Should be one zone
        self.assertEqual(1, response.json['counts'][0]['zones'])
        # Should be 1 NS and 1 SOA records
        self.assertEqual(2, response.json['counts'][0]['records'])
        # Should be 1 tenant
        self.assertEqual(1, response.json['counts'][0]['tenants'])

    def test_get_counts_zones(self):
        self.policy({'count_zones': '@'})
        response = self.client.get('/reports/counts/zones')

        self.assertEqual(200, response.status_int)
        self.assertEqual('application/json', response.content_type)

        self.assertIn('counts', response.json)
        self.assertIn('zones', response.json['counts'][0])

        self.assertEqual(0, response.json['counts'][0]['zones'])

        # Create 2 zones
        self.create_zone(fixture=0)
        self.create_zone(fixture=1)

        response = self.client.get('/reports/counts/zones')

        self.assertEqual(2, response.json['counts'][0]['zones'])

    def test_get_counts_zones_delayed_notify(self):
        # Count zones that are pending a NOTIFY transaction
        self.policy({'count_zones_delayed_notify': '@'})
        response = self.client.get('/reports/counts/zones_delayed_notify')

        self.assertEqual(200, response.status_int)
        self.assertEqual('application/json', response.content_type)
        self.assertIn('counts', response.json)
        self.assertIn('zones_delayed_notify', response.json['counts'][0])
        self.assertEqual(0, response.json['counts'][0]['zones_delayed_notify'])

        # Create 2 zones in pending notify and 1 with delayed_notify=False
        self.create_zone(fixture=0, delayed_notify=True)
        self.create_zone(fixture=1, delayed_notify=True)
        self.create_zone(fixture=2)

        response = self.client.get('/reports/counts/zones')
        self.assertEqual(3, response.json['counts'][0]['zones'])

        response = self.client.get('/reports/counts/zones_delayed_notify')
        self.assertEqual(2, response.json['counts'][0]['zones_delayed_notify'])

    def test_get_counts_records(self):
        self.policy({'count_records': '@'})
        response = self.client.get('/reports/counts/records')

        self.assertEqual(200, response.status_int)
        self.assertEqual('application/json', response.content_type)

        self.assertIn('counts', response.json)
        self.assertIn('records', response.json['counts'][0])

        self.assertEqual(0, response.json['counts'][0]['records'])

        # Create a zone
        self.create_zone()

        response = self.client.get('/reports/counts/records')

        # Should have 1 NS and 1 SOA record
        self.assertEqual(2, response.json['counts'][0]['records'])

    def test_get_counts_tenants(self):
        self.policy({'count_tenants': '@'})
        response = self.client.get('/reports/counts/tenants')

        self.assertEqual(200, response.status_int)
        self.assertEqual('application/json', response.content_type)

        self.assertIn('counts', response.json)
        self.assertIn('tenants', response.json['counts'][0])

        self.assertEqual(0, response.json['counts'][0]['tenants'])

        # Create a zone
        self.create_zone()

        response = self.client.get('/reports/counts/tenants')

        # Should have 1 tenant
        self.assertEqual(1, response.json['counts'][0]['tenants'])

    def test_get_tenants(self):
        self.policy({'find_tenants': '@'})
        self.create_zone()

        response = self.client.get('/reports/tenants')

        self.assertEqual(200, response.status_int)
        self.assertEqual('application/json', response.content_type)

        self.assertIn('tenants', response.json)
        self.assertIn('zone_count', response.json['tenants'][0])

        self.assertEqual(1, response.json['tenants'][0]['zone_count'])

    def test_get_tenants_detailed(self):
        self.policy({'find_tenants': '@'})
        self.create_zone(fixture=0)
        self.create_zone(fixture=1)

        response = self.client.get('/reports/tenants?detail=yes')

        self.assertEqual(200, response.status_int)
        self.assertEqual('application/json', response.content_type)

        self.assertIn('tenants', response.json)
        self.assertIn('zones_count', response.json['tenants'][0])

        self.assertEqual(2, response.json['tenants'][0]['zones_count'])

    def test_get_tenant(self):
        self.policy({'find_tenants': '@'})
        zone = self.create_zone()
        project_id = zone.tenant_id

        response = self.client.get('/reports/tenants/%s' % project_id)
        self.assertEqual(200, response.status_int)
        self.assertEqual('application/json', response.content_type)

        self.assertIn('zones', response.json)

        self.assertIn('example.com.', response.json['zones'])



# Copyright 2012 Hewlett-Packard Development Company, L.P. All Rights Reserved.
#
# Author: Kiall Mac Innes <kiall@hpe.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import flask

from designate.common import constants
import designate.conf


CONF = designate.conf.CONF


def _add_a_version(versions, version, api_url, status, timestamp):
    versions.append({
        'id': version,
        'status': status,
        'updated': timestamp,
        'links': [{'href': api_url,
                   'rel': 'self'},
                  {'href': constants.API_REF_URL,
                   'rel': 'help'}]
    })


def factory(global_config, **local_conf):
    app = flask.Flask('designate.api.versions')

    @app.route('/', methods=['GET'])
    def version_list():
        if CONF['service:api'].enable_host_header:
            url_root = flask.request.url_root
        else:
            url_root = CONF['service:api'].api_base_uri
        api_url = url_root.rstrip('/') + '/v2'

        versions = []
        # Initial API version for v2 API
        _add_a_version(versions, 'v2', api_url, constants.SUPPORTED,
                       '2022-06-29T00:00:00Z')
        _add_a_version(versions, 'v2.0', api_url, constants.SUPPORTED,
                       '2022-06-29T00:00:00Z')
        # 2.1 Shared Zones
        _add_a_version(versions, 'v2.1', api_url, constants.CURRENT,
                       '2023-01-25T00:00:00Z')

        return flask.jsonify({'versions': versions})

    return app



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.mport threading


from unittest import mock

from oslo_config import fixture as cfg_fixture
import oslotest.base

from designate.common import profiler
import designate.conf
from designate import policy
from designate import rpc
from designate.sink import service
from designate.tests import base_fixtures


CONF = designate.conf.CONF


class TestSinkService(oslotest.base.BaseTestCase):
    @mock.patch.object(policy, 'init')
    @mock.patch.object(rpc, 'get_client')
    @mock.patch.object(profiler, 'setup_profiler')
    def setUp(self, mock_setup_profiler, mock_get_client, mock_policy_init):
        super().setUp()
        self.stdlog = base_fixtures.StandardLogging()
        self.useFixture(self.stdlog)
        self.useFixture(cfg_fixture.Config(CONF))

        CONF.set_override(
            'enabled_notification_handlers', ['fake'], 'service:sink'
        )
        CONF.set_override(
            'allowed_event_types', ['compute.instance.create.end'],
            'handler:fake'
        )

        self.service = service.Service()

        self.context = mock.Mock()

        mock_setup_profiler.assert_called()
        mock_get_client.assert_called()
        mock_policy_init.assert_called()

    @mock.patch.object(rpc, 'get_notification_listener')
    def test_service_start(self, mock_notification_listener):
        self.service.start()

        mock_notification_listener.assert_called()

    @mock.patch.object(policy, 'init', mock.Mock())
    @mock.patch.object(rpc, 'get_client', mock.Mock())
    @mock.patch.object(profiler, 'setup_profiler', mock.Mock())
    @mock.patch.object(designate.rpc, 'get_notification_listener')
    def test_service_start_no_targets(self, mock_notification_listener):
        CONF.set_override(
            'enabled_notification_handlers', [], 'service:sink'
        )

        sink_service = service.Service()

        sink_service.start()

        mock_notification_listener.assert_not_called()

    def test_service_stop(self):
        self.service._notification_listener = None

        self.service.stop()

        self.assertIn('Stopping sink service', self.stdlog.logger.output)
        self.assertIsNone(self.service._notification_listener)

    def test_service_stop_and_notification_listener_stopped(self):
        self.service._notification_listener = mock.Mock()

        self.service.stop()

        self.assertIn('Stopping sink service', self.stdlog.logger.output)
        self.service._notification_listener.stop.assert_called_with()

    def test_service_name(self):
        self.assertEqual('sink', self.service.service_name)

    def test_get_allowed_event_types(self):
        self.assertEqual(
            ['compute.instance.create.end'],
            self.service.get_allowed_event_types(self.service.handlers)
        )

    def test_get_allowed_event_types_with_duplicates(self):
        mock_handler1 = mock.Mock()
        mock_handler2 = mock.Mock()

        mock_handler1.get_event_types.return_value = [
            'compute.instance.create.start'
        ]
        mock_handler2.get_event_types.return_value = [
            'compute.instance.create.end',
            'compute.instance.create.start'
        ]

        handlers = [mock_handler1, mock_handler2]

        self.assertEqual(
            ['compute.instance.create.start', 'compute.instance.create.end'],
            self.service.get_allowed_event_types(handlers)
        )

    def test_service_info(self):
        events = [
            'compute.instance.create.end',
            'compute.instance.create.start'
        ]
        self.service.allowed_event_types = events

        for event in events:
            self.service.info(
                self.context, 'publisher_id', event, mock.Mock(), mock.Mock()
            )

        self.assertIn(
            'received notification - compute.instance.create.end',
            self.stdlog.logger.output
        )
        self.assertNotIn(
            'received notification - compute.instance.create.start',
            self.stdlog.logger.output
        )



#!/usr/bin/env python3
# Copyright 2014 Hewlett-Packard Development Company, L.P.
#
# Author: Kiall Mac Innes <kiall@hp.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import binascii
import socket

# Bind to UDP 5355
sock_udp = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
sock_udp.bind(('', 5355))

# Receive a Packet
payload, addr = sock_udp.recvfrom(8192)

# Print the hex representation of the packet
print(binascii.b2a_hex(payload))

# The request just happens to be a kinda-ish valid response
sock_udp.sendto(payload, addr)



# Copyright 2016 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import random

from oslo_log import log as logging

from designate.objects import PoolList
from designate.scheduler.filters import base

LOG = logging.getLogger(__name__)


class RandomFilter(base.Filter):
    """Randomly chooses one of the input pools if there are multiple
    ones supplied.

    .. note::

        This should be used as one of the last filters, as it reduces the
        supplied pool list to one.
    """
    name = 'random'
    """Name to enable in the ``[designate:central:scheduler].filters`` option
    list
    """

    def filter(self, context, pools, zone):
        if pools:
            new_pool_list = PoolList()
            new_pool_list.append(random.choice(pools))
            return new_pool_list
        return pools



# Copyright 2014 Hewlett-Packard Development Company, L.P.
#
# Author: Kiall Mac Innes <kiall@hpe.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from oslo_config import cfg

DEFAULT_MDNS_PORT = 5354

MDNS_GROUP = cfg.OptGroup(
    name='service:mdns', title='Configuration for mDNS Service'
)

MDNS_OPTS = [
    cfg.IntOpt('workers',
               help='Number of mDNS worker processes to spawn'),
    cfg.IntOpt('threads', default=1000,
               help='Number of mDNS greenthreads to spawn'),
    cfg.ListOpt('listen',
                default=['0.0.0.0:%d' % DEFAULT_MDNS_PORT],
                help='mDNS host:port pairs to listen on'),
    cfg.IntOpt('tcp_backlog', default=100,
               help='mDNS TCP Backlog'),
    cfg.IntOpt('tcp_keepidle',
               help='mDNS TCP Keepidle in seconds'),
    cfg.FloatOpt('tcp_recv_timeout', default=0.5,
                 help='mDNS TCP Receive Timeout in seconds'),
    cfg.BoolOpt('query_enforce_tsig', default=False,
                help='Enforce all incoming queries (including AXFR) are TSIG '
                     'signed'),
    cfg.IntOpt('max_message_size', default=65535,
               help='Maximum message size to emit'),
]


def register_opts(conf):
    conf.register_group(MDNS_GROUP)
    conf.register_opts(MDNS_OPTS, group=MDNS_GROUP)


def list_opts():
    return {
        MDNS_GROUP: MDNS_OPTS
    }



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from unittest import mock

import oslotest.base

from designate.api.v2.controllers.zones import recordsets
from designate.central import rpcapi
from designate import exceptions
from designate import objects


class TestRecordsetAPI(oslotest.base.BaseTestCase):
    def setUp(self):
        super().setUp()
        self.central_api = mock.Mock()
        self.zone = objects.Zone(
            id='1e8952a5-e5a4-426a-afab-4cd10131a351',
            name='example.com.',
            email='example@example.com'
        )
        mock.patch.object(rpcapi.CentralAPI, 'get_instance',
                          return_value=self.central_api).start()

        self.controller = recordsets.RecordSetsController()

    @mock.patch('pecan.response', mock.Mock())
    @mock.patch('pecan.request')
    def test_post_all_soa_not_allowed(self, mock_request):
        mock_request.environ = {'context': mock.Mock()}
        mock_request.body_dict = {
            'name': 'soa.example.com.',
            'type': 'SOA'
        }

        self.assertRaisesRegex(
            exceptions.BadRequest,
            'Creating a SOA recordset is not allowed',
            self.controller.post_all, self.zone.id
        )

    @mock.patch('pecan.response')
    @mock.patch('pecan.request')
    def test_put_one(self, mock_request, mock_response):
        mock_context = mock.Mock()
        mock_context.edit_managed_records = False

        mock_request.environ = {'context': mock_context}

        record_set = objects.RecordSet(name='www.example.org.', type='NS')
        record_set.records = objects.RecordList(
            objects=[objects.Record(data='ns1.example.org.', action='NONE')]
        )

        self.central_api.get_recordset.return_value = record_set
        self.central_api.update_recordset.return_value = record_set
        self.central_api.get_zone.return_value = self.zone

        self.controller.put_one(
            self.zone.id, '99a60ad0-b9ac-4e83-9eee-859e99299bcf'
        )
        self.assertEqual(200, mock_response.status_int)

    @mock.patch('pecan.response', mock.Mock())
    @mock.patch('pecan.request')
    def test_put_one_managed_not_allowed(self, mock_request):
        mock_context = mock.Mock()
        mock_context.edit_managed_records = False

        mock_request.environ = {'context': mock_context}

        record_set = objects.RecordSet(name='www.example.org.', type='A')
        record_set.records = objects.RecordList(
            objects=[objects.Record(data='192.0.2.1', managed=True)]
        )

        self.central_api.get_recordset.return_value = record_set

        self.assertRaisesRegex(
            exceptions.BadRequest,
            'Managed records may not be updated',
            self.controller.put_one, self.zone.id,
            '3a2a2c3a-8f47-4788-8622-231f1c8f19c3'
        )

    @mock.patch('pecan.response', mock.Mock())
    @mock.patch('pecan.request')
    def test_put_one_soa_not_allowed(self, mock_request):
        mock_context = mock.Mock()

        mock_request.environ = {'context': mock_context}

        record_set = objects.RecordSet(name='soa.example.org.', type='SOA')
        record_set.records = objects.RecordList(
            objects=[objects.Record(data='192.0.2.2', managed=True)]
        )

        self.central_api.get_recordset.return_value = record_set

        self.assertRaisesRegex(
            exceptions.BadRequest,
            'Updating SOA recordsets is not allowed',
            self.controller.put_one, self.zone.id,
            '3a2a2c3a-8f47-4788-8622-231f1c8f19c3'
        )

    @mock.patch('pecan.response', mock.Mock())
    @mock.patch('pecan.request')
    def test_put_one_update_root_ns_not_allowed(self, mock_request):
        mock_context = mock.Mock()

        mock_request.environ = {'context': mock_context}

        record_set = objects.RecordSet(name='example.com.', type='NS')
        record_set.records = objects.RecordList(
            objects=[objects.Record(data='192.0.2.3', managed=True)]
        )

        self.central_api.get_recordset.return_value = record_set
        self.central_api.get_zone.return_value = self.zone

        self.assertRaisesRegex(
            exceptions.BadRequest,
            'Updating a root zone NS record is not allowed',
            self.controller.put_one, self.zone.id,
            '3a2a2c3a-8f47-4788-8622-231f1c8f19c3'
        )

    @mock.patch('pecan.response', mock.Mock())
    @mock.patch('pecan.request')
    def test_delete_one_soa_not_allowed(self, mock_request):
        mock_request.environ = {'context': mock.Mock()}

        record_set = objects.RecordSet(name='soa.example.com.', type='SOA')
        record_set.records = objects.RecordList(
            objects=[objects.Record(data='192.0.2.4')]
        )

        self.central_api.get_recordset.return_value = record_set

        self.assertRaisesRegex(
            exceptions.BadRequest,
            'Deleting a SOA recordset is not allowed',
            self.controller.delete_one, self.zone.id,
            '3a2a2c3a-8f47-4788-8622-231f1c8f19c3'
        )



# Copyright 2014 eBay Inc.
#
# Author: Ron Rickard <rrickard@ebay.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from oslo_log import log as logging

from designate.backend import base


LOG = logging.getLogger(__name__)


class FakeBackend(base.Backend):
    __plugin_name__ = 'fake'

    def create_zone(self, context, zone):
        LOG.info('Create Zone %r', zone)

    def delete_zone(self, context, zone, zone_params=None):
        LOG.info('Delete Zone %r', zone)



# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


from oslo_log import versionutils
from oslo_policy import policy

from designate.common import constants
from designate.common.policies import base

DEPRECATED_REASON = """
The zone API now supports system scope and default roles.
"""

deprecated_create_zone = policy.DeprecatedRule(
    name="create_zone",
    check_str=base.RULE_ADMIN_OR_OWNER,
    deprecated_reason=DEPRECATED_REASON,
    deprecated_since=versionutils.deprecated.WALLABY
)
deprecated_get_zones = policy.DeprecatedRule(
    name="get_zones",
    check_str=base.RULE_ADMIN_OR_OWNER,
    deprecated_reason=DEPRECATED_REASON,
    deprecated_since=versionutils.deprecated.WALLABY
)
deprecated_get_zone = policy.DeprecatedRule(
    name="get_zone",
    check_str=base.RULE_ADMIN_OR_OWNER_OR_SHARED,
    deprecated_reason=DEPRECATED_REASON,
    deprecated_since=versionutils.deprecated.WALLABY
)
deprecated_get_zone_servers = policy.DeprecatedRule(
    name="get_zone_servers",
    check_str=base.RULE_ADMIN_OR_OWNER,
    deprecated_reason=DEPRECATED_REASON,
    deprecated_since=versionutils.deprecated.WALLABY
)
deprecated_get_zone_ns_records = policy.DeprecatedRule(
    name="get_zone_ns_records",
    check_str=base.RULE_ADMIN_OR_OWNER,
    deprecated_reason=DEPRECATED_REASON,
    deprecated_since=versionutils.deprecated.WALLABY
)
deprecated_find_zones = policy.DeprecatedRule(
    name="find_zones",
    check_str=base.RULE_ADMIN_OR_OWNER,
    deprecated_reason=DEPRECATED_REASON,
    deprecated_since=versionutils.deprecated.WALLABY
)
deprecated_update_zone = policy.DeprecatedRule(
    name="update_zone",
    check_str=base.RULE_ADMIN_OR_OWNER,
    deprecated_reason=DEPRECATED_REASON,
    deprecated_since=versionutils.deprecated.WALLABY
)
deprecated_delete_zone = policy.DeprecatedRule(
    name="delete_zone",
    check_str=base.RULE_ADMIN_OR_OWNER,
    deprecated_reason=DEPRECATED_REASON,
    deprecated_since=versionutils.deprecated.WALLABY
)
deprecated_xfr_zone = policy.DeprecatedRule(
    name="xfr_zone",
    check_str=base.RULE_ADMIN_OR_OWNER,
    deprecated_reason=DEPRECATED_REASON,
    deprecated_since=versionutils.deprecated.WALLABY
)
deprecated_abandon_zone = policy.DeprecatedRule(
    name="abandon_zone",
    check_str=base.RULE_ADMIN,
    deprecated_reason=DEPRECATED_REASON,
    deprecated_since=versionutils.deprecated.WALLABY
)
deprecated_count_zones = policy.DeprecatedRule(
    name="count_zones",
    check_str=base.RULE_ADMIN_OR_OWNER,
    deprecated_reason=DEPRECATED_REASON,
    deprecated_since=versionutils.deprecated.WALLABY
)
deprecated_count_zones_pending_notify = policy.DeprecatedRule(
    name="count_zones_pending_notify",
    check_str=base.RULE_ADMIN_OR_OWNER,
    deprecated_reason=DEPRECATED_REASON,
    deprecated_since=versionutils.deprecated.WALLABY
)
deprecated_purge_zones = policy.DeprecatedRule(
    name="purge_zones",
    check_str=base.RULE_ADMIN,
    deprecated_reason=DEPRECATED_REASON,
    deprecated_since=versionutils.deprecated.WALLABY
)
deprecated_pool_move_zone = policy.DeprecatedRule(
    name="pool_move_zone",
    check_str=base.RULE_ADMIN,
    deprecated_reason=DEPRECATED_REASON,
    deprecated_since=versionutils.deprecated.WALLABY
)


rules = [
    policy.DocumentedRuleDefault(
        name="create_zone",
        check_str=base.SYSTEM_ADMIN_OR_PROJECT_MEMBER,
        scope_types=[constants.PROJECT],
        description="Create Zone",
        operations=[
            {
                'path': '/v2/zones',
                'method': 'POST'
            }
        ],
        deprecated_rule=deprecated_create_zone
    ),
    policy.RuleDefault(
        name="get_zones",
        check_str=base.SYSTEM_OR_PROJECT_READER,
        scope_types=[constants.PROJECT],
        deprecated_rule=deprecated_get_zones
    ),
    policy.DocumentedRuleDefault(
        name="get_zone",
        check_str=base.SYSTEM_OR_PROJECT_READER_OR_SHARED,
        scope_types=[constants.PROJECT],
        description="Get Zone",
        operations=[
            {
                'path': '/v2/zones/{zone_id}',
                'method': 'GET'
            }
        ],
        deprecated_rule=deprecated_get_zone
    ),
    policy.RuleDefault(
        name="get_zone_servers",
        check_str=base.SYSTEM_OR_PROJECT_READER,
        scope_types=[constants.PROJECT],
        deprecated_rule=deprecated_get_zone_servers
    ),
    policy.DocumentedRuleDefault(
        name="get_zone_ns_records",
        check_str=base.SYSTEM_OR_PROJECT_READER,
        scope_types=[constants.PROJECT],
        description="Get the Name Servers for a Zone",
        operations=[
            {
                'path': '/v2/zones/{zone_id}/nameservers',
                'method': 'GET'
            }
        ],
        deprecated_rule=deprecated_get_zone_ns_records
    ),
    policy.DocumentedRuleDefault(
        name="find_zones",
        check_str=base.SYSTEM_OR_PROJECT_READER,
        scope_types=[constants.PROJECT],
        description="List existing zones",
        operations=[
            {
                'path': '/v2/zones',
                'method': 'GET'
            }
        ],
        deprecated_rule=deprecated_find_zones
    ),
    policy.DocumentedRuleDefault(
        name="update_zone",
        check_str=base.SYSTEM_ADMIN_OR_PROJECT_MEMBER,
        scope_types=[constants.PROJECT],
        description="Update Zone",
        operations=[
            {
                'path': '/v2/zones/{zone_id}',
                'method': 'PATCH'
            }
        ],
        deprecated_rule=deprecated_update_zone
    ),
    policy.DocumentedRuleDefault(
        name="delete_zone",
        check_str=base.SYSTEM_ADMIN_OR_PROJECT_MEMBER,
        scope_types=[constants.PROJECT],
        description="Delete Zone",
        operations=[
            {
                'path': '/v2/zones/{zone_id}',
                'method': 'DELETE'
            }
        ],
        deprecated_rule=deprecated_delete_zone
    ),
    policy.DocumentedRuleDefault(
        name="xfr_zone",
        check_str=base.SYSTEM_ADMIN_OR_PROJECT_MEMBER,
        scope_types=[constants.PROJECT],
        description="Manually Trigger an Update of a Secondary Zone",
        operations=[
            {
                'path': '/v2/zones/{zone_id}/tasks/xfr',
                'method': 'POST'
            }
        ],
        deprecated_rule=deprecated_xfr_zone
    ),
    policy.DocumentedRuleDefault(
        name="abandon_zone",
        check_str=base.SYSTEM_ADMIN,
        scope_types=[constants.PROJECT],
        description="Abandon Zone",
        operations=[
            {
                'path': '/v2/zones/{zone_id}/tasks/abandon',
                'method': 'POST'
            }
        ],
        deprecated_rule=deprecated_abandon_zone
    ),
    policy.RuleDefault(
        name="count_zones",
        check_str=base.SYSTEM_OR_PROJECT_READER,
        scope_types=[constants.PROJECT],
        deprecated_rule=deprecated_count_zones
    ),
    policy.RuleDefault(
        name="count_zones_pending_notify",
        check_str=base.SYSTEM_OR_PROJECT_READER,
        scope_types=[constants.PROJECT],
        deprecated_rule=deprecated_count_zones_pending_notify
    ),
    policy.RuleDefault(
        name="purge_zones",
        check_str=base.SYSTEM_ADMIN,
        scope_types=[constants.PROJECT],
        deprecated_rule=deprecated_purge_zones
    ),
    policy.DocumentedRuleDefault(
        name="pool_move_zone",
        check_str=base.SYSTEM_ADMIN,
        scope_types=[constants.PROJECT],
        description="Pool Move Zone",
        operations=[
            {
                'path': '/v2/zones/{zone_id}/tasks/pool_move',
                'method': 'POST'
            }
        ],
        deprecated_rule=deprecated_pool_move_zone,
    )
]


def list_rules():
    return rules



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import os
from unittest import mock
import yaml

from oslo_log import log as logging
import oslo_messaging

from designate.central import service
from designate import exceptions
from designate.manage import base
from designate.manage import pool
from designate.tests import base_fixtures
import designate.tests.functional
from designate.tests import resources

LOG = logging.getLogger(__name__)


def get_pools_path(name='pools.yaml'):
    return os.path.join(resources.path, 'pools_yaml', name)


def get_pools(name='pools.yaml'):
    with open(get_pools_path(name)) as pool_obj:
        return yaml.safe_load(pool_obj)


class ManagePoolTestCase(designate.tests.functional.TestCase):
    def setUp(self):
        super().setUp()
        self.stdlog = base_fixtures.StandardLogging()
        self.useFixture(self.stdlog)

        default_pool = self.central_service.find_pool(
            self.admin_context, {'name': 'default'}
        )
        self.central_service.delete_pool(self.admin_context, default_pool.id)

        self.command = pool.PoolCommands()

        self.print_result = mock.patch.object(
            base.Commands, '_print_result').start()

    def test_show_config(self):
        self.command._setup()
        self.command._create_pool(get_pools()[0])

        pool_id = self.central_service.find_pool(
            self.admin_context, {'name': 'default'}).id

        self.command.show_config(pool_id, all_pools=False)

        self.print_result.assert_called_once()
        self.assertIn('Pool Configuration', self.command.output_message[1])
        self.assertIn(
            'Default PowerDNS 4 Pool', ''.join(self.command.output_message)
        )

    def test_show_config_catalog_zone(self):
        self.command._setup()
        self.command._create_pool(get_pools('pools-catalog-zone.yaml')[0])

        pool_id = self.central_service.find_pool(
            self.admin_context, {'name': 'default'}).id

        self.command.show_config(pool_id, all_pools=False)

        self.print_result.assert_called_once()
        self.assertIn('Pool Configuration', self.command.output_message[1])
        self.assertIn(
            'Default PowerDNS 4 Pool', ''.join(self.command.output_message)
        )

    @mock.patch.object(service.Service, 'find_pool',
                       side_effect=oslo_messaging.MessagingTimeout())
    def test_show_config_rpc_timeout(self, mock_find_pool):
        self.assertRaises(
            SystemExit,
            self.command.show_config, '5421ca70-f1b7-4edc-9e01-b604011a262a',
            all_pools=False
        )

        mock_find_pool.assert_called_once()

    def test_show_config_pool_not_found(self):
        self.assertRaises(
            SystemExit,
            self.command.show_config, '5421ca70-f1b7-4edc-9e01-b604011a262a',
            all_pools=False
        )
        self.assertIn(
            'Pool not found', ''.join(self.command.output_message)
        )

    def test_show_config_invalid_uuid(self):
        self.assertRaises(
            SystemExit,
            self.command.show_config, 'None', all_pools=False
        )
        self.print_result.assert_called_once()
        self.assertIn(
            'Not a valid uuid: None', ''.join(self.command.output_message)
        )

    def test_show_config_empty(self):
        self.assertRaises(
            SystemExit,
            self.command.show_config, 'a36bb018-9584-420c-acc6-2b5cf89714ad',
            all_pools=False
        )
        self.print_result.assert_called_once()
        self.assertIn('Pool not found', ''.join(self.command.output_message))

    def test_show_config_multiple_pools(self):
        self.command._setup()
        self.command._create_pool(get_pools(name='multiple-pools.yaml')[0])
        self.command._create_pool(get_pools(name='multiple-pools.yaml')[1])

        # Calling show_config --all_pools without specifying pool_id
        self.command.show_config(None, all_pools=True)

        self.print_result.assert_called_once()

        pools = self.central_service.find_pools(self.admin_context, {})
        self.assertIn('Pool Configuration', self.command.output_message[1])
        for p in pools:
            self.assertIn(p.id, ''.join(self.command.output_message))
            self.assertIn(p.description,
                          ''.join(self.command.output_message))

        # Calling show_config --all_pools with pool_id
        # (should ignore the pool_id)
        self.command.show_config('a36bb018-9584-420c-acc6-2b5cf89714ad',
                                 all_pools=True)
        for p in pools:
            self.assertEqual(2, sum(
                p.id in s for s in self.command.output_message))
            self.assertEqual(2, sum(
                p.description in s for s in self.command.output_message))

    def test_update(self):
        self.command.update(
            get_pools_path('pools.yaml'), delete=False, dry_run=False
        )

        self.print_result.assert_called_once()
        self.assertIn(
            'Updating Pools Configuration****************************',
            ''.join(self.command.output_message)
        )

        pool = self.central_service.find_pool(self.admin_context, {
            'name': 'default'
        })

        self.assertEqual(1, len(pool.targets))
        self.assertEqual('pdns4', pool.targets[0].type)

    def test_update_bind9(self):
        self.command.update(
            get_pools_path('bind9_pools.yaml'), delete=False, dry_run=False
        )

        self.print_result.assert_called_once()
        self.assertIn(
            'Updating Pools Configuration****************************',
            ''.join(self.command.output_message)
        )

        pool = self.central_service.find_pool(self.admin_context, {
            'name': 'bind'
        })

        self.assertEqual(1, len(pool.targets))
        self.assertEqual('bind9', pool.targets[0].type)

    def test_update_multiple_pools(self):
        self.command.update(
            get_pools_path('multiple-pools.yaml'), delete=False, dry_run=False
        )

        self.print_result.assert_called_once()
        self.assertIn(
            'Updating Pools Configuration****************************',
            ''.join(self.command.output_message)
        )

        pools = self.central_service.find_pools(self.admin_context, {})
        self.assertEqual(2, len(pools))

    def test_update_multiple_pools_name(self):
        self.command.update(
            get_pools_path('pools.yaml'), delete=False, dry_run=False
        )

        pools = self.central_service.find_pools(self.admin_context, {})
        self.assertEqual(1, len(pools))

        # Updating an existing pool (same name) to a different id should fail
        self.assertRaises(
            exceptions.DuplicatePool,
            self.command.update,
            get_pools_path('sample_output.yaml'), delete=False, dry_run=False
        )

        pools = self.central_service.find_pools(self.admin_context, {})
        self.assertEqual(1, len(pools))

        # Updating Pools with different name will only add pools
        self.command.update(
            get_pools_path('multiple-pools.yaml'), delete=False, dry_run=False
        )

        pools = self.central_service.find_pools(self.admin_context, {})
        self.assertEqual(3, len(pools))

    @mock.patch.object(service.Service, 'find_pool',
                       side_effect=oslo_messaging.MessagingTimeout())
    def test_update_rpc_timeout(self, mock_find_pool):
        self.assertRaises(
            SystemExit,
            self.command.update,
            get_pools_path('pools.yaml'), delete=False, dry_run=False
        )

        mock_find_pool.assert_called_once()

    @mock.patch.object(pool.PoolCommands, '_load_config')
    def test_update_pool_with_invalid_uuid(self, mock_load_config):
        mock_load_config.return_value = [{
            'name': 'default',
            'id': 'invalid',
        }]

        self.assertRaises(
            SystemExit,
            self.command.update, 'test.yaml', delete=False, dry_run=False
        )
        self.assertIn(
            'Not a valid uuid: invalid',
            ''.join(self.command.output_message)
        )

    @mock.patch.object(pool.PoolCommands, '_load_config')
    def test_update_pool_invalid_ns_record(self, mock_load_config):
        mock_load_config.return_value = [{
            'name': 'default',
            'ns_records': [
                {'hostname': 'ns1-1.example.org.', 'priority': None},
            ],
            'targets': [
                {
                    'type': 'powerdns',
                }
            ],
        }]

        self.assertRaises(
            SystemExit,
            self.command.update, 'test.yaml', delete=False, dry_run=False
        )
        self.assertIn(
            "Provided object is not valid. Got a ValueError error with "
            "message Field `priority' cannot be None",
            ''.join(self.command.output_message)
        )

    @mock.patch.object(pool.PoolCommands, '_load_config')
    def test_update_new_backend(self, mock_load_config):
        self.command._setup()
        self.command._create_pool(get_pools()[0])

        self.create_zone(fixture=0)
        self.create_zone(fixture=1)

        pools = self.central_service.find_pools(self.admin_context, {})
        self.assertEqual(1, len(pools))
        self.assertEqual('pdns4', pools[0].targets[0].type)

        new_default = dict(get_pools()[0])
        new_default['targets'][0]['type'] = 'bind9'

        mock_load_config.return_value = [new_default]

        self.command.update('test.yaml', delete=False, dry_run=False)

        mock_load_config.assert_called_once_with('test.yaml')
        self.print_result.assert_called_once()
        self.assertIn(
            'Updating Pools Configuration****************************',
            ''.join(self.command.output_message)
        )

        pools = self.central_service.find_pools(self.admin_context, {})
        self.assertEqual(1, len(pools))
        self.assertEqual('bind9', pools[0].targets[0].type)

    @mock.patch.object(pool.PoolCommands, '_load_config')
    def test_update_pool_unknown_backend(self, mock_load_config):
        mock_load_config.return_value = [{
            'name': 'default',
            'ns_records': [
                {'hostname': 'ns1-1.example.org.', 'priority': 1},
            ],
            'targets': [
                {
                    'type': 'powerdns',
                }
            ],
        }]

        self.assertRaises(
            SystemExit,
            self.command.update, 'test.yaml', delete=False, dry_run=False
        )
        self.assertIn(
            'Unable to find designate backend driver type: powerdns',
            ''.join(self.command.output_message)
        )

    @mock.patch.object(pool.PoolCommands, '_load_config')
    def test_update_pool_unknown_backend_dry_run(self, mock_load_config):
        mock_load_config.return_value = [{
            'name': 'default',
            'ns_records': [
            ],
            'targets': [
                {
                    'type': 'powerdns',
                }
            ],
        }]

        self.command.update(
            'test.yaml', delete=False, dry_run=True
        )

        self.assertIn(
            'Unable to find designate backend driver type: powerdns',
            ''.join(self.command.output_message)
        )

    @mock.patch.object(pool.PoolCommands, '_load_config')
    def test_update_pool_unknown_backend_skip_verify(self, mock_load_config):
        mock_load_config.return_value = [{
            'name': 'default',
            'ns_records': [
            ],
            'targets': [
                {
                    'type': 'powerdns',
                }
            ],
        }]

        self.command.update(
            'test.yaml', delete=False, dry_run=False, skip_verify_drivers=True
        )

        self.assertNotIn(
            'Unable to find designate backend driver type: powerdns',
            ''.join(self.command.output_message)
        )

    def test_update_with_delete(self):
        self.command.update(
            get_pools_path('multiple-pools.yaml'), delete=True, dry_run=False
        )

        pools = self.central_service.find_pools(self.admin_context, {})
        self.assertEqual(2, len(pools))

        self.command.update(
            get_pools_path('pools.yaml'), delete=True, dry_run=False
        )

        self.print_result.assert_called()

        pools = self.central_service.find_pools(self.admin_context, {})
        self.assertEqual(1, len(pools))

    @mock.patch.object(pool.PoolCommands, '_load_config')
    def test_update_with_delete_dry_run(self, mock_load_config):
        default_pool = dict(get_pools()[0])
        default_pool['id'] = 'a234253f-9fd8-4e1c-996e-5bcb152f43d5'
        additional_pool = {
            'name': 'second_pool',
            'ns_records': [
                {'hostname': 'ns1-1.example.org.', 'priority': 1},
            ],
            'targets': [
                {
                    'type': 'pdns4',
                }
            ],
        }

        mock_load_config.return_value = get_pools()

        self.command._setup()
        self.command._create_pool(default_pool)
        self.command._create_pool(additional_pool)

        self.command.update('test.yaml', delete=True, dry_run=True)

        mock_load_config.assert_called_once_with('test.yaml')
        self.print_result.assert_called_once()
        self.assertIn(
            "Update Pool: <Pool id:'a234253f-9fd8-4e1c-996e-5bcb152f43d5' "
            "name:'default'>",
            ' '.join(self.command.output_message)
        )
        self.assertIn(
            'Delete Pool: second_pool',
            ''.join(self.command.output_message)
        )

        pools = self.central_service.find_pools(self.admin_context, {})
        self.assertEqual(2, len(pools))

    @mock.patch.object(pool.PoolCommands, '_load_config')
    def test_update_dry_run(self, mock_load_config):
        mock_load_config.return_value = get_pools()

        self.command.update('test.yaml', delete=True, dry_run=True)

        mock_load_config.assert_called_once_with('test.yaml')
        self.print_result.assert_called_once()
        self.assertIn(
            "Create Pool: <Pool id:'None' name:'default'>",
            ''.join(self.command.output_message)
        )

    @mock.patch.object(pool.PoolCommands, '_write_config_to_file')
    def test_generate_file(self, mock_write_config_to_file):
        self.command._setup()
        self.command._create_pool(get_pools()[0])

        self.command.generate_file('test.yaml')

        mock_write_config_to_file.assert_called_once()

    @mock.patch.object(service.Service, 'find_pools',
                       side_effect=oslo_messaging.MessagingTimeout())
    def test_generate_file_rpc_timeout(self, mock_find_pools):
        self.assertRaises(
            SystemExit,
            self.command.generate_file, 'test.yaml'
        )

        mock_find_pools.assert_called_once()

    def test_create_new_pool(self):
        pools = self.central_service.find_pools(self.admin_context, {})
        self.assertEqual(0, len(pools))

        self.command._setup()
        self.command._create_pool(get_pools()[0])

        new_pool = self.central_service.find_pool(
            self.admin_context, {'name': 'default'}
        )

        self.assertEqual('default', new_pool.name)
        self.assertEqual('Default PowerDNS 4 Pool', new_pool.description)

        pools = self.central_service.find_pools(self.admin_context, {})
        self.assertEqual(1, len(pools))

    def test_get_pool_by_id(self):
        self.command._setup()
        self.command._create_pool(get_pools()[0])

        new_pool = self.central_service.find_pool(
            self.admin_context, {'name': 'default'}
        )

        self.assertEqual(
            'default',
            self.command._get_pool({'id': new_pool.id}).name
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from unittest import mock

import oslo_messaging as messaging
import oslotest.base

import designate.conf
from designate import rpc

CONF = designate.conf.CONF


def action_test(a, b):
    return f'{a}-{b}'


class DesignateObjectSerializerTest(oslotest.base.BaseTestCase):
    def test_process_iterable(self):
        context = 'context'
        values = ['foo', 'bar']

        serializer = rpc.DesignateObjectSerializer()
        self.assertEqual(
            ['context-bar', 'context-foo'],
            sorted(serializer._process_iterable(context, action_test, values))
        )

    def test_process_iterable_with_set(self):
        context = 'context'
        values = {'foo', 'bar'}

        serializer = rpc.DesignateObjectSerializer()
        self.assertEqual(
            ('context-bar', 'context-foo'),
            tuple(sorted(
                serializer._process_iterable(context, action_test, values))
            )
        )


class RequestContextSerializerTest(oslotest.base.BaseTestCase):
    def test_serialize_entity_base_not_set(self):
        entity = 'entity'

        request = rpc.RequestContextSerializer(None)
        self.assertEqual(entity, request.serialize_entity(None, entity))

    def test_deserialize_entity_base_not_set(self):
        entity = 'entity'

        request = rpc.RequestContextSerializer(None)
        self.assertEqual(entity, request.deserialize_entity(None, entity))

    @mock.patch.object(rpc, 'profiler', mock.Mock())
    def test_serialize_context_with_profiler(self):
        mock_context = mock.Mock()
        mock_context.to_dict.return_value = {}
        request = rpc.RequestContextSerializer(None)

        self.assertIn('trace_info', request.serialize_context(mock_context))

    @mock.patch.object(rpc, 'profiler', None)
    def test_serialize_context_without_profiler(self):
        mock_context = mock.Mock()
        mock_context.to_dict.return_value = {}
        request = rpc.RequestContextSerializer(None)

        self.assertNotIn('trace_info', request.serialize_context(mock_context))

    @mock.patch.object(rpc, 'profiler')
    @mock.patch('designate.context.DesignateContext.from_dict', mock.Mock())
    def test_deserialize_context_with_profiler(self, mock_profile):
        mock_context = mock.Mock()
        mock_context.pop.return_value = {'key': 'value'}
        request = rpc.RequestContextSerializer(None)

        request.deserialize_context(mock_context)

        mock_profile.init.assert_called_with(key='value')

    @mock.patch.object(rpc, 'profiler')
    @mock.patch('designate.context.DesignateContext.from_dict', mock.Mock())
    def test_deserialize_context_without_trace_info(self, mock_profile):
        mock_context = mock.Mock()
        mock_context.pop.return_value = None
        request = rpc.RequestContextSerializer(None)

        request.deserialize_context(mock_context)

        mock_profile.init.assert_not_called()


class RPCTest(oslotest.base.BaseTestCase):
    @mock.patch.object(designate.rpc, 'NOTIFIER', mock.Mock())
    @mock.patch.object(designate.rpc, 'NOTIFICATION_TRANSPORT')
    @mock.patch.object(designate.rpc, 'TRANSPORT')
    def test_cleanup(self, mock_transport, mock_notification_transport):
        self.assertTrue(rpc.TRANSPORT)
        self.assertTrue(rpc.NOTIFICATION_TRANSPORT)
        self.assertTrue(rpc.NOTIFIER)

        rpc.cleanup()

        mock_transport.cleanup.assert_called_once()
        mock_notification_transport.cleanup.assert_called_once()

        self.assertIsNone(rpc.TRANSPORT)
        self.assertIsNone(rpc.NOTIFICATION_TRANSPORT)
        self.assertIsNone(rpc.NOTIFIER)

    @mock.patch.object(designate.rpc, 'NOTIFIER', True)
    @mock.patch.object(designate.rpc, 'NOTIFICATION_TRANSPORT', True)
    @mock.patch.object(designate.rpc, 'TRANSPORT', None)
    def test_cleanup_no_transport(self):
        self.assertRaisesRegex(
            AssertionError,
            r"'TRANSPORT' must not be None",
            rpc.cleanup
        )

    @mock.patch.object(designate.rpc, 'NOTIFIER', True)
    @mock.patch.object(designate.rpc, 'NOTIFICATION_TRANSPORT', None)
    @mock.patch.object(designate.rpc, 'TRANSPORT', True)
    def test_cleanup_no_notification_transport(self):
        self.assertRaisesRegex(
            AssertionError,
            r"'NOTIFICATION_TRANSPORT' must not be None",
            rpc.cleanup
        )

    @mock.patch.object(designate.rpc, 'NOTIFIER', None)
    @mock.patch.object(designate.rpc, 'NOTIFICATION_TRANSPORT', True)
    @mock.patch.object(designate.rpc, 'TRANSPORT', True)
    def test_cleanup_no_notifier(self):
        self.assertRaisesRegex(
            AssertionError,
            r"'NOTIFIER' must not be None",
            rpc.cleanup
        )

    def test_add_extra_exmods(self):
        rpc.add_extra_exmods('arg1', 'arg2', 'arg3')
        self.assertEqual(['arg1', 'arg2', 'arg3'], rpc.EXTRA_EXMODS)

        rpc.clear_extra_exmods()

    def test_clear_extra_exmods(self):
        rpc.add_extra_exmods('arg1', 'arg2', 'arg3')
        rpc.clear_extra_exmods()

        self.assertEqual([], rpc.EXTRA_EXMODS)

    @mock.patch.object(messaging, 'set_transport_defaults')
    def test_set_defaults(self, mock_set_transport_defaults):
        rpc.set_defaults('test')

        mock_set_transport_defaults.assert_called_with('test')

    @mock.patch.object(designate.rpc, 'NOTIFIER', True)
    @mock.patch.object(designate.rpc, 'NOTIFICATION_TRANSPORT', True)
    @mock.patch.object(designate.rpc, 'TRANSPORT', True)
    @mock.patch.object(messaging, 'get_rpc_server')
    def test_get_server(self, mock_get_rpc_server):
        rpc.get_server(None, None, True)

        mock_get_rpc_server.assert_called_with(
            True, None, None,
            executor='eventlet', serializer=mock.ANY,
            access_policy=mock.ANY,
        )

    @mock.patch.object(designate.rpc, 'TRANSPORT', None)
    def test_get_server_transport_is_none(self):
        self.assertRaisesRegex(
            AssertionError,
            r"'TRANSPORT' must not be None",
            rpc.get_server, None, None
        )

    @mock.patch.object(designate.rpc, 'TRANSPORT', True)
    @mock.patch.object(messaging, 'get_rpc_client')
    def test_get_client(self, mock_get_rpc_client):
        rpc.get_client(None, None, True)

        mock_get_rpc_client.assert_called_with(
            True, None,
            version_cap=None, serializer=mock.ANY,
        )

    @mock.patch.object(designate.rpc, 'TRANSPORT', None)
    def test_get_client_is_none(self):
        self.assertRaisesRegex(
            AssertionError,
            r"'TRANSPORT' must not be None",
            rpc.get_client, None, None
        )

    @mock.patch.object(designate.rpc, 'NOTIFICATION_TRANSPORT', True)
    @mock.patch.object(messaging, 'get_notification_listener')
    def test_get_notification_listener(self, mock_get_notification_listener):
        rpc.get_notification_listener('target', 'endpoint', serializer=None)

        mock_get_notification_listener.assert_called_with(
            True, 'target', 'endpoint', executor='eventlet', pool=None,
            serializer=mock.ANY
        )

    @mock.patch.object(designate.rpc, 'NOTIFICATION_TRANSPORT', True)
    @mock.patch.object(messaging, 'get_notification_listener')
    def test_get_notification_listener_serializer_set(self,
                                             mock_get_notification_listener):
        rpc.get_notification_listener('target', 'endpoint', serializer=True)

        mock_get_notification_listener.assert_called_with(
            True, 'target', 'endpoint', executor='eventlet', pool=None,
            serializer=True
        )

    @mock.patch.object(designate.rpc, 'NOTIFICATION_TRANSPORT', None)
    def test_get_notification_listener_transport_is_none(self):
        self.assertRaisesRegex(
            AssertionError,
            r"'NOTIFICATION_TRANSPORT' must not be None",
            rpc.get_notification_listener, None, None
        )

    @mock.patch.object(designate.rpc, 'NOTIFIER', None)
    @mock.patch.object(designate.rpc, 'NOTIFICATION_TRANSPORT', None)
    @mock.patch.object(designate.rpc, 'TRANSPORT', None)
    @mock.patch('oslo_messaging.notify.notifier.Notifier.prepare')
    def test_get_notifier(self, mock_prepare):
        rpc.init(CONF)

        rpc.get_notifier(publisher_id=True)

        mock_prepare.assert_called_with(publisher_id=True)

    @mock.patch.object(designate.rpc, 'NOTIFIER', None)
    def test_get_notifier_is_none(self):
        self.assertRaisesRegex(
            AssertionError,
            r"'NOTIFIER' must not be None",
            rpc.get_notifier
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from oslo_log import log as logging
import oslotest.base

from designate.common.decorators import rpc
import designate.conf
from designate.tests import base_fixtures


CONF = designate.conf.CONF
LOG = logging.getLogger(__name__)


@rpc.rpc_logging(LOG, 'test')
class RPCLog:
    RPC_LOGGING_DISALLOW = ['ignore']

    @classmethod
    def test(cls):
        return None

    @classmethod
    def ignore(cls):
        return None


class TestRPCLogging(oslotest.base.BaseTestCase):
    def setUp(self):
        super().setUp()
        self.stdlog = base_fixtures.StandardLogging()
        self.useFixture(self.stdlog)

    def test_rpc_logging(self):
        RPCLog.test()

        self.assertIn(
            'Calling designate.test.test() over RPC', self.stdlog.logger.output
        )

    def test_rpc_disallow_logging(self):
        RPCLog.ignore()

        self.assertFalse(self.stdlog.logger.output)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from unittest import mock

from oslo_log import log as logging
from oslo_utils import timeutils

import designate.conf
from designate.tests import base_fixtures
import designate.tests.functional
from designate.worker import processing
from designate.worker.tasks import zone


CONF = designate.conf.CONF
LOG = logging.getLogger(__name__)


class RecoverShardTest(designate.tests.functional.TestCase):
    def setUp(self):
        super().setUp()
        self.stdlog = base_fixtures.StandardLogging()
        self.useFixture(self.stdlog)
        self.executor = processing.Executor()

    def test_recover_error_create(self):
        new_zone = self.create_zone(shard=9)
        new_zone.status = 'ERROR'
        self.storage.update_zone(self.admin_context, new_zone)
        self.create_zone(fixture=1, shard=10)

        self.task = zone.RecoverShard(
            self.executor, self.admin_context, 0, new_zone.shard
        )

        self.task._worker_api = mock.Mock()

        self.task()

        self.task.worker_api.create_zone.assert_called()
        self.task.worker_api.update_zone.assert_not_called()
        self.task.worker_api.delete_zone.assert_not_called()

    def test_recover_error_update(self):
        new_zone = self.create_zone(shard=9)
        new_zone.action = 'UPDATE'
        new_zone.status = 'ERROR'
        self.storage.update_zone(self.admin_context, new_zone)
        self.create_zone(fixture=1, shard=10)

        self.task = zone.RecoverShard(
            self.executor, self.admin_context, 0, new_zone.shard
        )

        self.task._worker_api = mock.Mock()

        self.task()

        self.task.worker_api.create_zone.assert_not_called()
        self.task.worker_api.update_zone.assert_called()
        self.task.worker_api.delete_zone.assert_not_called()

    def test_recover_error_delete(self):
        new_zone = self.create_zone(shard=9)
        new_zone.action = 'DELETE'
        new_zone.status = 'ERROR'
        self.storage.update_zone(self.admin_context, new_zone)

        second_zone = self.create_zone(fixture=1, shard=10)
        second_zone.action = 'DELETE'
        second_zone.status = 'ERROR'
        self.storage.update_zone(self.admin_context, second_zone)

        self.task = zone.RecoverShard(
            self.executor, self.admin_context, 0, new_zone.shard
        )

        self.task._worker_api = mock.Mock()

        self.task()

        self.task.worker_api.create_zone.assert_not_called()
        self.task.worker_api.update_zone.assert_not_called()
        self.task.worker_api.delete_zone.assert_called()

    def test_recover_stale(self):
        new_zone = self.create_zone(
            shard=9,
        )
        new_zone.serial = timeutils.utcnow_ts() - 1000
        new_zone.action = 'DELETE'
        new_zone.status = 'PENDING'
        self.storage.update_zone(self.admin_context, new_zone)
        self.create_zone(fixture=1, shard=5)

        self.task = zone.RecoverShard(
            self.executor, self.admin_context, 0, new_zone.shard
        )

        self.task._worker_api = mock.Mock()

        self.task()

        self.task.worker_api.create_zone.assert_not_called()
        self.task.worker_api.update_zone.assert_not_called()
        self.task.worker_api.delete_zone.assert_called()

    def test_recover_nothing_to_do(self):
        self.create_zone(shard=0)
        self.create_zone(fixture=1, shard=2048)
        self.create_zone(fixture=2, shard=4095)

        self.task = zone.RecoverShard(
            self.executor, self.admin_context, 0, 4095
        )

        self.task._worker_api = mock.Mock()

        self.task()

        self.task.worker_api.create_zone.assert_not_called()
        self.task.worker_api.update_zone.assert_not_called()
        self.task.worker_api.delete_zone.assert_not_called()

    def test_recover_no_valid_actions(self):
        new_zone = self.create_zone(shard=9)
        new_zone.action = 'NONE'
        new_zone.status = 'ERROR'
        self.storage.update_zone(self.admin_context, new_zone)
        self.create_zone(fixture=1, shard=2048)
        self.create_zone(fixture=2, shard=4095)

        self.task = zone.RecoverShard(
            self.executor, self.admin_context, 0, 4095
        )

        self.task._worker_api = mock.Mock()

        self.task()

        self.task.worker_api.create_zone.assert_not_called()
        self.task.worker_api.update_zone.assert_not_called()
        self.task.worker_api.delete_zone.assert_not_called()



# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


from oslo_log import versionutils
from oslo_policy import policy

from designate.common import constants
from designate.common.policies import base

DEPRECATED_REASON = """
The tenant API now supports system scope and default roles.
"""

deprecated_find_tenants = policy.DeprecatedRule(
    name="find_tenants",
    check_str=base.RULE_ADMIN,
    deprecated_reason=DEPRECATED_REASON,
    deprecated_since=versionutils.deprecated.WALLABY
)
deprecated_get_tenant = policy.DeprecatedRule(
    name="get_tenant",
    check_str=base.RULE_ADMIN,
    deprecated_reason=DEPRECATED_REASON,
    deprecated_since=versionutils.deprecated.WALLABY
)
deprecated_count_tenants = policy.DeprecatedRule(
    name="count_tenants",
    check_str=base.RULE_ADMIN,
    deprecated_reason=DEPRECATED_REASON,
    deprecated_since=versionutils.deprecated.WALLABY
)


rules = [
    policy.RuleDefault(
        name="find_tenants",
        check_str=base.SYSTEM_READER,
        scope_types=[constants.PROJECT],
        description="Find all Tenants.",
        deprecated_rule=deprecated_find_tenants
    ),
    policy.RuleDefault(
        name="get_tenant",
        check_str=base.SYSTEM_READER,
        scope_types=[constants.PROJECT],
        description="Get all Tenants.",
        deprecated_rule=deprecated_get_tenant
    ),
    policy.RuleDefault(
        name="count_tenants",
        check_str=base.SYSTEM_READER,
        scope_types=[constants.PROJECT],
        description="Count tenants",
        deprecated_rule=deprecated_count_tenants
    )
]


def list_rules():
    return rules



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import random
from unittest import mock

import jinja2
from oslo_concurrency import processutils
from oslo_config import fixture as cfg_fixture
import oslotest.base

import designate.conf
from designate import exceptions
from designate.tests import base_fixtures
from designate import utils


CONF = designate.conf.CONF


class TestUtils(oslotest.base.BaseTestCase):
    def setUp(self):
        super().setUp()
        self.stdlog = base_fixtures.StandardLogging()
        self.useFixture(cfg_fixture.Config(CONF))
        self.useFixture(self.stdlog)

    def test_validate_uuid(self):
        @utils.validate_uuid('zone_id')
        def validate_uuid(cls, zone_id):
            return True
        self.assertTrue(
            validate_uuid(None, '6a8caf9d-679e-4fc1-80e9-13496f6a783f')
        )

    def test_validate_invalid_uuid(self):
        @utils.validate_uuid('zone_id')
        def validate_uuid(cls, zone_id):
            return True

        self.assertRaisesRegex(
            exceptions.InvalidUUID,
            'Invalid UUID zone_id: 62f89e5f088c7',
            validate_uuid, None, '62f89e5f088c7'
        )

    def test_validate_uuid_no_arguments(self):
        @utils.validate_uuid('zone_id')
        def validate_uuid(cls):
            return
        self.assertRaises(exceptions.NotFound, validate_uuid)

    def test_invalid_uuid_no_argument_provided(self):
        @utils.validate_uuid('zone_id')
        def validate_uuid(cls):
            return
        self.assertRaises(exceptions.NotFound, validate_uuid, None)

    @mock.patch('os.path.exists')
    @mock.patch('os.path.abspath')
    def test_find_config(self, mock_abspath, mock_path_exists):
        CONF.set_override('pybasedir', '/tmp/workspace/designate')

        mock_path_exists.side_effect = [True, False, False, False, False]
        mock_abspath.return_value = '/tmp/designate/designate.conf'

        config_files = utils.find_config('designate.conf')

        self.assertEqual(['/tmp/designate/designate.conf'], config_files)
        mock_abspath.assert_called_once()

    @mock.patch.object(processutils, 'execute')
    def test_execute(self, mock_execute):
        mock_execute.return_value = ('designate.conf\npools.yaml\n', '')

        out, err = utils.execute(
            '/bin/ls', '/etc/designate/',
            run_as_root=False
        )

        mock_execute.assert_called_once_with(
            '/bin/ls', '/etc/designate/',
            root_helper='sudo designate-rootwrap /etc/designate/rootwrap.conf',
            run_as_root=False
        )

        self.assertEqual('designate.conf\npools.yaml\n', out)
        self.assertFalse(err)

    @mock.patch.object(processutils, 'execute')
    def test_execute_with_rootwrap(self, mock_execute):
        CONF.set_override('root_helper', 'sudo designate-test')

        mock_execute.return_value = ('designate.conf\npools.yaml\n', '')

        out, err = utils.execute(
            '/bin/ls', '/etc/designate/',
            run_as_root=True
        )

        mock_execute.assert_called_once_with(
            '/bin/ls', '/etc/designate/',
            root_helper='sudo designate-test',
            run_as_root=True
        )

        self.assertEqual('designate.conf\npools.yaml\n', out)
        self.assertFalse(err)

    def test_deep_dict_merge(self):
        a = {
            'a': {'dns': 'record'},
            'b': 'b',
            'c': 'c',
        }

        b = {
            'a': {'domain': 'zone'},
            'c': 1,
            'd': 'd',
        }

        self.assertEqual(
            {
                'a': {
                    'dns': 'record', 'domain': 'zone'
                },
                'b': 'b', 'c': 1, 'd': 'd'
            },
            utils.deep_dict_merge(a, b)
        )

    def test_deep_dict_merge_not_dict(self):
        result = utils.deep_dict_merge(dict(), list())

        self.assertIsInstance(result, list)

    def test_get_proxies(self):
        CONF.set_override('no_proxy', 'example.com', 'proxy')
        CONF.set_override('http_proxy', 'example.org', 'proxy')
        CONF.set_override('https_proxy', 'example.net', 'proxy')

        result = utils.get_proxies()

        self.assertEqual(['example.com'], result.get('no_proxy'))
        self.assertEqual('example.org', result.get('http'))
        self.assertEqual('example.net', result.get('https'))

    def test_get_proxies_default_values(self):
        result = utils.get_proxies()

        self.assertIsNone(result.get('no_proxy'))
        self.assertIsNone(result.get('http'))
        self.assertIsNone(result.get('https'))

    def test_get_proxies_with_no_proxy(self):
        CONF.set_override('no_proxy', 'example.org', 'proxy')

        result = utils.get_proxies()

        self.assertEqual(['example.org'], result.get('no_proxy'))
        self.assertIsNone(result.get('http'))
        self.assertIsNone(result.get('https'))

    def test_get_proxies_with_http_proxy(self):
        CONF.set_override('http_proxy', 'example.org', 'proxy')

        result = utils.get_proxies()

        self.assertIsNone(result.get('no_proxy'))
        self.assertEqual('example.org', result.get('http'))
        self.assertEqual('example.org', result.get('https'))

    def test_get_proxies_with_https_proxy(self):
        CONF.set_override('https_proxy', 'example.org', 'proxy')

        result = utils.get_proxies()

        self.assertIsNone(result.get('no_proxy'))
        self.assertIsNone(result.get('http'))
        self.assertEqual('example.org', result.get('https'))

    def test_resource_string(self):
        resource_string = utils.resource_string(
            'templates', 'bind9-zone.jinja2'
        )

        self.assertIsNotNone(resource_string)

    def test_resource_string_missing(self):
        self.assertRaisesRegex(
            exceptions.ResourceNotFound,
            'Could not find the requested resource',
            utils.resource_string, 'invalid.jinja2'
        )

    def test_resource_string_empty_args(self):
        self.assertRaises(
            ValueError,
            utils.resource_string
        )

    def test_load_schema_missing(self):
        self.assertRaisesRegex(
            exceptions.ResourceNotFound,
            'Could not find the requested resource',
            utils.load_schema, 'v1', 'missing'
        )

    @mock.patch.object(utils, 'resource_string')
    def test_load_template(self, mock_resource_string):
        mock_resource_string.return_value = b'Hello {{name}}'

        template = utils.load_template('bind9-zone.jinja2')

        self.assertIsInstance(template, jinja2.Template)

    @mock.patch.object(utils, 'resource_string')
    def test_load_template_keep_trailing_newline(self, mock_resource_string):
        mock_resource_string.return_value = b'Hello {{name}}'

        template = utils.load_template('bind9-zone.jinja2')

        self.assertTrue(template.environment.keep_trailing_newline)

    def test_load_template_missing(self):
        self.assertRaises(
            exceptions.ResourceNotFound,
            utils.load_template, 'invalid.jinja2'
        )

    def test_render_template(self):
        template = jinja2.Template('Hello {{name}}')

        result = utils.render_template(template, name='World')

        self.assertEqual('Hello World', result)

    def test_split_host_port(self):
        host, port = utils.split_host_port('abc:25')
        self.assertEqual(('abc', 25), (host, port))

    def test_split_host_port_with_invalid_port(self):
        host, port = utils.split_host_port('abc:abc')
        self.assertEqual(('abc:abc', 53), (host, port))

    def test_get_paging_params(self):
        CONF.set_override('default_limit_v2', 100, 'service:api')

        context = mock.Mock()
        params = {
            'updated_at': None,
            'created_at': '2019-06-28T04:17:34.000000',
            'pattern': 'blacklisted.com.',
            'id': 'f6663a98-281e-4cea-b0c3-3bc425e086ea',
        }

        marker, limit, sort_key, sort_dir = utils.get_paging_params(
            context, params, ['created_at', 'id', 'updated_at', 'pattern']
        )

        self.assertIsNone(marker)
        self.assertEqual(100, limit)
        self.assertIsNone(sort_key)
        self.assertIsNone(sort_dir)

    def test_get_paging_params_without_sort_keys(self):
        CONF.set_override('default_limit_v2', 0, 'service:api')

        context = mock.Mock()
        params = {
            'updated_at': None,
            'created_at': '2019-06-28T04:17:34.000000',
            'pattern': 'blacklisted.com.',
            'id': 'f6663a98-281e-4cea-b0c3-3bc425e086ea',
        }

        marker, limit, sort_key, sort_dir = utils.get_paging_params(
            context, params, sort_keys=None
        )

        self.assertIsNone(marker)
        self.assertEqual(0, limit)
        self.assertIsNone(sort_key)
        self.assertIsNone(sort_dir)

    def test_get_paging_params_sort_by_tenant_id(self):
        CONF.set_override('default_limit_v2', 100, 'service:api')

        context = mock.Mock()
        context.all_tenants = True
        params = {
            'updated_at': None,
            'created_at': '2019-06-28T04:17:34.000000',
            'pattern': 'blacklisted.com.',
            'id': 'f6663a98-281e-4cea-b0c3-3bc425e086ea',
            'sort_key': 'tenant_id',
        }

        marker, limit, sort_key, sort_dir = utils.get_paging_params(
            context, params,
            ['created_at', 'id', 'updated_at', 'pattern', 'tenant_id']
        )

        self.assertIsNone(marker)
        self.assertEqual(100, limit)
        self.assertEqual('tenant_id', sort_key)
        self.assertIsNone(sort_dir)

    def test_get_paging_params_sort_tenant_without_all_tenants(self):
        CONF.set_override('default_limit_v2', 100, 'service:api')

        context = mock.Mock()
        context.all_tenants = False
        params = {
            'updated_at': None,
            'created_at': '2019-06-28T04:17:34.000000',
            'pattern': 'blacklisted.com.',
            'id': 'f6663a98-281e-4cea-b0c3-3bc425e086ea',
            'sort_key': 'tenant_id',
        }

        marker, limit, sort_key, sort_dir = utils.get_paging_params(
            context, params,
            ['created_at', 'id', 'updated_at', 'pattern', 'tenant_id']
        )

        self.assertIsNone(marker)
        self.assertEqual(100, limit)
        self.assertIsNone(sort_key)
        self.assertIsNone(sort_dir)

    def test_get_paging_params_invalid_limit(self):
        context = mock.Mock()

        self.assertRaises(
            exceptions.InvalidLimit,
            utils.get_paging_params,
            context, {'limit': 9223372036854775809}, []
        )

        self.assertRaises(
            exceptions.InvalidLimit,
            utils.get_paging_params,
            context, {'limit': -1}, []
        )

    def test_get_paging_params_max_limit(self):
        CONF.set_override('max_limit_v2', 1000, 'service:api')

        context = mock.Mock()

        result = utils.get_paging_params(context, {'limit': 'max'}, [])

        self.assertEqual(result[1], 1000)

    def test_get_paging_params_invalid_sort_dir(self):
        context = mock.Mock()

        self.assertRaisesRegex(
            exceptions.InvalidSortDir,
            'Unknown sort direction, must be',
            utils.get_paging_params, context, {'sort_dir': 'dsc'}, []
        )

    def test_get_paging_params_invalid_sort_key(self):
        context = mock.Mock()

        self.assertRaisesRegex(
            exceptions.InvalidSortKey,
            'sort key must be one of',
            utils.get_paging_params, context, {'sort_key': 'dsc'},
            ['asc', 'desc']
        )

    @mock.patch('socket.socket')
    def test_bind_tcp(self, mock_sock_impl):
        mock_sock = mock.MagicMock()
        mock_sock_impl.return_value = mock_sock

        utils.bind_tcp('203.0.113.1', 53, 100, 1)

        mock_sock.settimeout.assert_called_once_with(1)

        mock_sock.bind.assert_called_once_with(('203.0.113.1', 53))

        mock_sock.listen.assert_called_once_with(100)

        self.assertIn(
            'Opening TCP Listening Socket on 203.0.113.1:53',
            self.stdlog.logger.output
        )

    @mock.patch('socket.socket')
    def test_bind_tcp_without_port(self, mock_sock_impl):
        random_port = random.randint(1024, 65535)
        mock_sock = mock.MagicMock()
        mock_sock_impl.return_value = mock_sock

        mock_sock.getsockname.return_value = ('203.0.113.1', random_port)

        utils.bind_tcp('203.0.113.1', 0, 100, 1)

        self.assertIn(
            'Listening on TCP port %(port)d' % {'port': random_port},
            self.stdlog.logger.output
        )

    @mock.patch('socket.socket')
    def test_bind_tcp_without_reuse_port(self, mock_sock_impl):
        mock_sock = mock.MagicMock()
        mock_sock_impl.return_value = mock_sock
        mock_sock.setsockopt.side_effect = [None, None, AttributeError, None]

        utils.bind_tcp('203.0.113.1', 53, 100, 1)

        self.assertIn(
            'SO_REUSEPORT not available, ignoring.',
            self.stdlog.logger.output
        )

    @mock.patch('socket.socket')
    def test_bind_udp(self, mock_sock_impl):
        mock_sock = mock.MagicMock()
        mock_sock_impl.return_value = mock_sock

        utils.bind_udp('203.0.113.1', 53)

        mock_sock.settimeout.assert_called_once_with(1)

        mock_sock.bind.assert_called_once_with(('203.0.113.1', 53))

        self.assertIn(
            'Opening UDP Listening Socket on 203.0.113.1:53',
            self.stdlog.logger.output
        )

    @mock.patch('socket.socket')
    def test_bind_udp_without_port(self, mock_sock_impl):
        random_port = random.randint(1024, 65535)
        mock_sock = mock.MagicMock()
        mock_sock_impl.return_value = mock_sock

        mock_sock.getsockname.return_value = ('203.0.113.1', random_port)

        utils.bind_udp('203.0.113.1', 0)

        self.assertIn(
            'Listening on UDP port %(port)d' % {'port': random_port},
            self.stdlog.logger.output
        )

    @mock.patch('socket.socket')
    def test_bind_udp_without_reuse_port(self, mock_sock_impl):
        mock_sock = mock.MagicMock()
        mock_sock_impl.return_value = mock_sock
        mock_sock.setsockopt.side_effect = [None, AttributeError]

        utils.bind_udp('203.0.113.1', 53)

        self.assertIn(
            'SO_REUSEPORT not available, ignoring.',
            self.stdlog.logger.output
        )



# Copyright 2014 Hewlett-Packard Development Company, L.P.
#
# Author: Endre Karlson <endre.karlson@hpe.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from unittest import mock

from oslo_config import fixture as cfg_fixture
import oslotest.base
import requests_mock

from designate.backend import impl_dynect
import designate.conf
from designate import context
from designate import exceptions
from designate import objects
from designate.tests import base_fixtures

CONF = designate.conf.CONF

MASTERS = [
    '192.0.2.1'
]
CONTACT = 'jdoe@example.org'

LOGIN_SUCCESS = {
    'status': 'success',
    'data': {
        'token': 'foo',
        'version': '3.5.6'
    },
    'job_id': 1,
    'msgs': [
        {
            'INFO': 'login: Login successful',
            'SOURCE': 'BLL',
            'ERR_CD': None,
            'LVL': 'INFO'
        }
    ]
}

LOGOUT_SUCCESS = {
    'status': 'success',
    'data': {},
    'job_id': 1345964647,
    'msgs': [
        {
            'INFO': 'logout: Logout successful',
            'SOURCE': 'BLL',
            'ERR_CD': None,
            'LVL': 'INFO'
        }
    ]
}

ZONE_CREATE = {
    'contact_nickname': 'owner',
    'masters': ['192.0.2.1'],
    'token': 'foo',
    'zone': 'example.com',
}

ZONE_DELETE = {
    'token': 'foo',
    'zone': 'example.com',
}

INVALID_ZONE_DELETE_DATA = {
    'status': 'failure',
    'data': {},
    'job_id': 1326038394,
    'msgs': [
        {
            'INFO': 'delete: Zone not deleted',
            'SOURCE': 'BLL',
            'ERR_CD': None,
            'LVL': 'INFO'
        }
    ]
}

INVALID_MASTER_DATA = {
    'status': 'failure',
    'data': {},
    'job_id': 1326038394,
    'msgs': [
        {
            'INFO': 'master: IP address expected',
            'SOURCE': 'DYN',
            'ERR_CD': 'INVALID_DATA',
            'LVL': 'ERROR'
        },
        {
            'INFO': 'create: Zone not created',
            'SOURCE': 'BLL',
            'ERR_CD': None,
            'LVL': 'INFO'
        }
    ]
}

TARGET_EXISTS = {
    'status': 'failure',
    'data': {},
    'job_id': 1345944906,
    'msgs': [
        {
            'INFO': 'name: Name already exists',
            'SOURCE': 'BLL',
            'ERR_CD': 'TARGET_EXISTS',
            'LVL': 'ERROR'
        },
        {
            'INFO': 'create: You already have this zone.',
            'SOURCE': 'BLL',
            'ERR_CD': None,
            'LVL': 'INFO'
        }
    ]
}

ACTIVATE_SUCCESS = {
    'status': 'success',
    'data': {
        'active': 'L',
        'masters': MASTERS,
        'contact_nickname': CONTACT,
        'tsig_key_name': '',
        'zone': 'example.com'
    },
    'job_id': 1345944927,
    'msgs': [
        {
            'INFO': 'activate: Service activated',
            'SOURCE': 'BLL',
            'ERR_CD': None,
            'LVL': 'INFO'
        }
    ]
}


class DynClientTestsCase(oslotest.base.BaseTestCase):
    def setUp(self):
        super().setUp()

    def test_dyn_client_auth_error(self):
        client = impl_dynect.DynClient(
            'customer_name', 'user_name', 'password', 'timeout', 'timings'
        )
        client.token = 'fake'
        client._request = mock.Mock()
        client._request.side_effect = impl_dynect.DynClientAuthError()

        self.assertRaisesRegex(
            impl_dynect.DynClientAuthError,
            r'None \(HTTP None to None - None\) - None',
            client.request, 'POST', '/Test/'
        )


class DynECTTestsCase(oslotest.base.BaseTestCase):
    def setUp(self):
        super().setUp()

        self.useFixture(cfg_fixture.Config(CONF))
        self.stdlog = base_fixtures.StandardLogging()
        self.useFixture(self.stdlog)

        self.context = mock.Mock()
        self.admin_context = mock.Mock()
        mock.patch.object(
            context.DesignateContext, 'get_admin_context',
            return_value=self.admin_context).start()

        self.base_address = 'https://api.dynect.net:443/REST'
        self.zone = objects.Zone(
            id='e2bed4dc-9d01-11e4-89d3-123b93f75cba',
            name='example.com.',
            email='example@example.com',
        )
        self.target = {
            'id': '4588652b-50e7-46b9-b688-a9bad40a873e',
            'type': 'dyndns',
            'masters': [
                {'host': '192.0.2.1', 'port': 53}
            ],
            'options': [
                {'key': 'username', 'value': 'example'},
                {'key': 'password', 'value': 'secret'},
                {'key': 'customer_name', 'value': 'customer'},
                {'key': 'contact_nickname', 'value': 'customer'},
                {'key': 'tsig_key_name', 'value': 'tsig'},
            ],
        }

        self.backend = impl_dynect.DynECTBackend(
            objects.PoolTarget.from_dict(self.target)
        )

    def test_masters_only_allow_port_53(self):
        self.target['masters'] = [
            {'host': '192.0.2.1', 'port': 5354}
        ]
        self.assertRaisesRegex(
            exceptions.ConfigurationError,
            'DynECT only supports mDNS instances on port 53',
            impl_dynect.DynECTBackend,
            objects.PoolTarget.from_dict(self.target)
        )

    @requests_mock.mock()
    def test_create_zone(self, req_mock):
        req_mock.post(
            '%s/Session' % self.base_address,
            json=LOGIN_SUCCESS,
        )
        req_mock.delete(
            '%s/Session' % self.base_address,
            json=LOGOUT_SUCCESS,
        )

        req_mock.post(
            '%s/Secondary/example.com' % self.base_address,
            json=ZONE_CREATE,
            status_code=200,
        )
        req_mock.put(
            '%s/Secondary/example.com' % self.base_address,
            json=ACTIVATE_SUCCESS,
            status_code=200,
        )

        self.backend.create_zone(self.context, self.zone)

    @requests_mock.mock()
    def test_create_zone_with_timings(self, req_mock):
        CONF.set_override('timings', True, 'backend:dynect')

        req_mock.post(
            '%s/Session' % self.base_address,
            json=LOGIN_SUCCESS,
        )
        req_mock.delete(
            '%s/Session' % self.base_address,
            json=LOGOUT_SUCCESS,
        )

        req_mock.post(
            '%s/Secondary/example.com' % self.base_address,
            json=ZONE_CREATE,
            status_code=200,
        )
        req_mock.put(
            '%s/Secondary/example.com' % self.base_address,
            json=ACTIVATE_SUCCESS,
            status_code=200,
        )

        self.backend.create_zone(self.context, self.zone)

    @requests_mock.mock()
    def test_create_zone_missing_contact_and_tsig(self, req_mock):
        self.target['options'] = [
            {'key': 'username', 'value': 'example'},
            {'key': 'password', 'value': 'secret'},
        ]

        backend = impl_dynect.DynECTBackend(
            objects.PoolTarget.from_dict(self.target)
        )

        req_mock.post(
            '%s/Session' % self.base_address,
            json=LOGIN_SUCCESS,
        )
        req_mock.delete(
            '%s/Session' % self.base_address,
            json=LOGOUT_SUCCESS,
        )

        req_mock.post(
            '%s/Secondary/example.com' % self.base_address,
            json=ZONE_CREATE,
            status_code=200,
        )
        req_mock.put(
            '%s/Secondary/example.com' % self.base_address,
            json=ACTIVATE_SUCCESS,
            status_code=200,
        )

        backend.create_zone(self.context, self.zone)

    @requests_mock.mock()
    def test_create_zone_raise_dynclienterror(self, req_mock):
        # https://api.dynect.net:443/REST/Session
        req_mock.post(
            '%s/Session' % self.base_address,
            json=LOGIN_SUCCESS,
        )

        req_mock.post(
            '%s/Secondary/example.com' % self.base_address,
            json=INVALID_MASTER_DATA,
            status_code=400,
        )

        self.assertRaisesRegex(
            impl_dynect.DynClientError, 'Zone not created',
            self.backend.create_zone, self.context, self.zone,
        )

    @requests_mock.mock()
    def test_create_zone_duplicate_updates_existing(self, req_mock):
        req_mock.post(
            '%s/Session' % self.base_address,
            json=LOGIN_SUCCESS,
        )

        req_mock.delete(
            '%s/Session' % self.base_address,
            json=LOGIN_SUCCESS,
        )

        req_mock.post(
            '%s/Secondary/example.com' % self.base_address,
            json=TARGET_EXISTS,
            status_code=400,
        )

        req_mock.put(
            '%s/Secondary/example.com' % self.base_address,
            json=ACTIVATE_SUCCESS,
        )

        self.backend.create_zone(self.context, self.zone)

    @requests_mock.mock()
    def test_delete_zone(self, req_mock):
        req_mock.post(
            '%s/Session' % self.base_address,
            json=LOGIN_SUCCESS,
        )
        req_mock.delete(
            '%s/Session' % self.base_address,
            json=LOGOUT_SUCCESS,
        )

        req_mock.delete(
            '%s/Zone/example.com' % self.base_address,
            json=ZONE_DELETE,
            status_code=200,
        )
        req_mock.put(
            '%s/Secondary/example.com' % self.base_address,
            json=ACTIVATE_SUCCESS,
            status_code=200,
        )

        self.backend.delete_zone(self.context, self.zone)

    @requests_mock.mock()
    def test_delete_zone_not_found(self, req_mock):
        req_mock.post(
            '%s/Session' % self.base_address,
            json=LOGIN_SUCCESS,
        )
        req_mock.delete(
            '%s/Session' % self.base_address,
            json=LOGOUT_SUCCESS,
        )

        req_mock.delete(
            '%s/Zone/example.com' % self.base_address,
            json=INVALID_ZONE_DELETE_DATA,
            status_code=404
        )
        req_mock.put(
            '%s/Secondary/example.com' % self.base_address,
            json=ACTIVATE_SUCCESS,
            status_code=200,
        )

        self.backend.delete_zone(self.context, self.zone)

        self.assertIn(
            'Attempt to delete e2bed4dc-9d01-11e4-89d3-123b93f75cba / '
            'example.com. caused 404, ignoring.',
            self.stdlog.logger.output
        )

    @requests_mock.mock()
    def test_delete_zone_raise_dynclienterror(self, req_mock):
        req_mock.post(
            '%s/Session' % self.base_address,
            json=LOGIN_SUCCESS,
        )
        req_mock.delete(
            '%s/Session' % self.base_address,
            json=LOGOUT_SUCCESS,
        )

        req_mock.delete(
            '%s/Zone/example.com' % self.base_address,
            json=INVALID_ZONE_DELETE_DATA,
            status_code=400,
        )
        req_mock.put(
            '%s/Secondary/example.com' % self.base_address,
            json=ACTIVATE_SUCCESS,
            status_code=200,
        )

        self.assertRaisesRegex(
            impl_dynect.DynClientError, 'Zone not deleted',
            self.backend.delete_zone, self.context, self.zone,
        )

    @requests_mock.mock()
    def test_request(self, req_mock):
        req_mock.post(
            '%s/Session' % self.base_address,
            json=LOGIN_SUCCESS,
        )
        req_mock.post(
            'https://api.dynect.net:443/'
        )
        req_mock.post(
            'https://api.dynect.net:443/REST'
        )

        client = self.backend.get_client()

        self.assertEqual(200,
                         client._request(
                             'POST',
                             'https://api.dynect.net:443/REST').status_code
                         )
        self.assertEqual(200, client._request('POST', '').status_code)

    def test_error_from_response(self):
        error_data = dict(INVALID_ZONE_DELETE_DATA)
        mock_response = mock.Mock()
        mock_response.json.return_value = error_data
        error = impl_dynect.DynClientError().from_response(mock_response)

        self.assertEqual(error_data['job_id'], error.job_id)

    def test_error_from_response_login_failed(self):
        error_data = dict(INVALID_ZONE_DELETE_DATA)
        error_data['msgs'] = [
            {
                'INFO': 'login: foo',
                'SOURCE': 'BLL',
                'ERR_CD': None,
                'LVL': 'INFO'
            }
        ]
        mock_response = mock.Mock()
        mock_response.json.return_value = error_data

        self.assertRaisesRegex(
            impl_dynect.DynClientAuthError,
            'login: foo',
            impl_dynect.DynClientError().from_response, mock_response
        )

    def test_error_from_response_operation_failed(self):
        error_data = dict(INVALID_ZONE_DELETE_DATA)
        error_data['msgs'] = [
            {
                'INFO': 'Operation blocked',
                'SOURCE': 'BLL',
                'ERR_CD': None,
                'LVL': 'INFO'
            }
        ]
        mock_response = mock.Mock()
        mock_response.json.return_value = error_data

        self.assertRaisesRegex(
            impl_dynect.DynClientOperationBlocked,
            'Operation blocked',
            impl_dynect.DynClientError().from_response, mock_response
        )



# Copyright 2015 Infoblox Inc.
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from unittest import mock

from infoblox_client import connector as infoblox_connector
from infoblox_client import exceptions as infoblox_exceptions
from infoblox_client import objects as infoblox_objects
import oslotest.base
import requests_mock

from designate.backend import impl_infoblox
from designate import context
from designate import exceptions
from designate import objects
from designate.tests import base_fixtures


class InfobloxBackendTestCase(oslotest.base.BaseTestCase):
    def setUp(self):
        super().setUp()
        self.stdlog = base_fixtures.StandardLogging()
        self.useFixture(self.stdlog)

        self.project_id = 'f532f66e-0fea-4698-895c-bb7caef815ef'
        self.admin_context = mock.Mock()
        self.admin_context.project_id = self.project_id
        mock.patch.object(
            context.DesignateContext, 'get_admin_context',
            return_value=self.admin_context).start()

        self.zone = objects.Zone(
            id='e2bed4dc-9d01-11e4-89d3-123b93f75cba',
            name='example.com.',
            email='example@example.com',
        )
        self.base_address = 'https://192.0.2.1/wapi/v2.10/'
        self.dns_view = 'my_dns_view'
        self.network_view = 'my_net_view'
        self.ns_group = 'my_ns_group'

        self.target = {
            'id': '4588652b-50e7-46b9-b688-a9bad40a873e',
            'type': 'designate',
            'masters': [
                {'host': '192.0.2.1', 'port': 53},
            ],
            'options': [
                {'key': 'wapi_url', 'value': self.base_address},
                {'key': 'username', 'value': 'user'},
                {'key': 'password', 'value': 'secret'},
                {'key': 'dns_view', 'value': self.dns_view},
                {'key': 'network_view', 'value': self.network_view},
                {'key': 'ns_group', 'value': self.ns_group},
            ],
        }


class BasicInfobloxBackendTestCase(InfobloxBackendTestCase):
    def setUp(self):
        super().setUp()

        self.target['options'].append(
            {'key': 'multi_tenant', 'value': '0'},
        )

        self.backend = impl_infoblox.InfobloxBackend(
            objects.PoolTarget.from_dict(self.target)
        )

        self.backend.connection = mock.Mock()
        self.backend.infoblox = mock.Mock()

    @mock.patch.object(impl_infoblox, 'infoblox_connector', None)
    def test_no_library_installed(self):
        pool_target = objects.PoolTarget.from_dict(self.target)
        self.assertRaisesRegex(
            exceptions.Backend,
            'The infoblox-client library is not available',
            impl_infoblox.InfobloxBackend, pool_target
        )

    def test_get_options(self):
        self.assertEqual('my_dns_view', self.backend.dns_view)
        self.assertEqual('my_net_view', self.backend.network_view)
        self.assertEqual('my_ns_group', self.backend.ns_group)
        self.assertEqual('0', self.backend.multi_project)

    @mock.patch.object(infoblox_connector, 'Connector', mock.Mock())
    def test_backend_with_invalid_master_port(self):
        self.target['masters'] = [
            {'host': '192.0.2.1', 'port': 5354},
        ]
        pool_target = objects.PoolTarget.from_dict(self.target)
        self.assertRaisesRegex(
            exceptions.ConfigurationError,
            'Infoblox only supports mDNS instances on port 53',
            impl_infoblox.InfobloxBackend, pool_target
        )

    @mock.patch.object(infoblox_connector, 'Connector')
    def test_backend_with_host(self, mock_infoblox_connector):
        self.target['options'] = [
            {'key': 'wapi_host', 'value': '192.0.2.100'},
            {'key': 'wapi_version', 'value': '1'},
            {'key': 'username', 'value': 'user'},
            {'key': 'password', 'value': 'secret'},
        ]
        impl_infoblox.InfobloxBackend(
            objects.PoolTarget.from_dict(self.target)
        )
        mock_infoblox_connector.assert_called_with(
            {
                'host': '192.0.2.100',
                'username': 'user',
                'password': 'secret',
                'http_pool_connections': None,
                'http_pool_maxsize': None,
                'wapi_version': '1',
                'ssl_verify': None,
                'cert': None,
                'key': None
            }
        )

    @mock.patch.object(infoblox_connector, 'Connector')
    def test_backend_with_wapi_url(self, mock_infoblox_connector):
        impl_infoblox.InfobloxBackend(
            objects.PoolTarget.from_dict(self.target)
        )
        mock_infoblox_connector.assert_called_with(
            {
                'host': '192.0.2.1',
                'username': 'user',
                'password': 'secret',
                'http_pool_connections': None,
                'http_pool_maxsize': None,
                'wapi_version': '2.10',
                'ssl_verify': None,
                'cert': None,
                'key': None
            }
        )

    def test_is_multi_project(self):
        self.backend.multi_project = True
        self.assertTrue(self.backend.is_multi_project)

        self.backend.multi_project = 1
        self.assertTrue(self.backend.is_multi_project)

        self.backend.multi_project = '1'
        self.assertTrue(self.backend.is_multi_project)

        self.backend.multi_project = False
        self.assertFalse(self.backend.is_multi_project)

        self.backend.multi_project = 0
        self.assertFalse(self.backend.is_multi_project)

        self.backend.multi_project = '0'
        self.assertFalse(self.backend.is_multi_project)

    def test_parse_wapi_url(self):
        self.assertEqual(
            ('192.0.2.1', None),
            self.backend.parse_wapi_url('https://192.0.2.1/')
        )
        self.assertEqual(
            ('192.0.2.2', '1'),
            self.backend.parse_wapi_url('https://192.0.2.2/wapi/v1/')
        )
        self.assertEqual(
            ('192.0.2.3', '2.10'),
            self.backend.parse_wapi_url('https://192.0.2.3/wapi/v2.10/')
        )
        self.assertEqual(
            ('192.0.2.3:443', '2.10'),
            self.backend.parse_wapi_url('https://192.0.2.3:443/wapi/v2.10/')
        )

    def test_get_network_view(self):
        self.backend.connection.get_object.return_value = [{'name': 'fake'}]

        self.assertEqual('fake', self.backend.get_network_view('project_id'))

    def test_get_network_view_no_result(self):
        self.backend.connection.get_object.return_value = []

        self.assertIsNone(self.backend.get_network_view('project_id'))

    def test_get_or_create_network_view(self):
        mock_network_view = mock.Mock()
        mock_network_view.name = 'fake'

        self.backend.connection.get_object.return_value = []
        self.backend.infoblox.create_network_view.return_value = (
            mock_network_view
        )

        self.assertEqual(
            'fake', self.backend.get_or_create_network_view('project_id')
        )

    def test_get_or_create_network_view_not_found(self):
        self.backend.connection.get_object.return_value = [{'name': 'fake'}]

        self.assertEqual(
            'fake', self.backend.get_or_create_network_view('project_id')
        )

    def test_get_or_create_network_view_already_found(self):
        self.backend.connection.get_object.return_value = [{'name': 'fake'}]

        self.assertEqual(
            'fake', self.backend.get_or_create_network_view('project_id')
        )

    @mock.patch.object(infoblox_objects.DNSView, 'search')
    def test_get_or_create_dns_view(self, mock_search):
        mock_dns_view = mock.Mock()
        mock_dns_view.name = 'fake'

        mock_search.return_value = None
        self.backend.infoblox.create_dns_view.return_value = mock_dns_view

        self.assertEqual(
            'fake', self.backend.get_or_create_dns_view('net_view')
        )

    def test_get_or_create_dns_view_no_network_provided(self):
        self.assertFalse(self.backend.get_or_create_dns_view(None))

    @mock.patch.object(infoblox_objects.DNSView, 'search')
    def test_get_or_create_dns_view_not_found(self, mock_search):
        mock_search.return_value = None

        self.assertFalse(
            self.backend.get_or_create_dns_view(
                'net_view', create_if_missing=False
            )
        )

    @mock.patch.object(infoblox_objects.DNSView, 'search')
    def test_get_or_create_dns_view_already_found(self, mock_search):
        mock_dns_view = mock.Mock()
        mock_dns_view.name = 'fake'

        mock_search.return_value = mock_dns_view

        self.assertEqual(
            'fake', self.backend.get_or_create_dns_view('net_view')
        )

    @mock.patch.object(infoblox_objects, 'Grid', mock.Mock())
    def test_restart_if_needed_unable_to_restart(self):
        self.backend.connection.call_func.side_effect = (
            infoblox_exceptions.InfobloxException('')
        )

        self.backend.restart_if_needed()

        self.assertIn(
            'Unable to restart the infoblox dns service.',
            self.stdlog.logger.output
        )

    def test_create_zone(self):
        self.backend.restart_if_needed = mock.Mock()

        self.backend.create_zone(self.admin_context, self.zone)

        self.backend.infoblox.create_dns_zone.assert_called_with(
            dns_zone='example.com',
            dns_view='my_dns_view',
            zone_format='FORWARD',
            ns_group='my_ns_group'
        )
        self.backend.restart_if_needed.assert_called()

    def test_create_zone_handle_error(self):
        self.backend.infoblox.create_dns_zone.side_effect = (
            infoblox_exceptions.InfobloxTimeoutError('error')
        )

        self.assertRaisesRegex(
            exceptions.Backend,
            'Connection to NIOS timed out',
            self.backend.create_zone, self.admin_context, self.zone
        )

    def test_create_zone_ptr(self):
        zone = objects.Zone(
            id='e2bed4dc-9d01-11e4-89d3-123b93f75cba',
            name='example.in-addr.arpa.',
            email='example@example.com',
        )

        self.backend.restart_if_needed = mock.Mock()

        self.backend.create_zone(self.admin_context, zone)

        self.backend.infoblox.create_dns_zone.assert_called_with(
            dns_zone='example.in-addr.arpa',
            dns_view='my_dns_view',
            zone_format='IPV4',
            ns_group='my_ns_group'
        )
        self.backend.restart_if_needed.assert_called()

    def test_create_zone_ipv6_ptr(self):
        zone = objects.Zone(
            id='e2bed4dc-9d01-11e4-89d3-123b93f75cba',
            name='example.ip6.arpa.',
            email='example@example.com',
        )

        self.backend.restart_if_needed = mock.Mock()

        self.backend.create_zone(self.admin_context, zone)

        self.backend.infoblox.create_dns_zone.assert_called_with(
            dns_zone='example.ip6.arpa',
            dns_view='my_dns_view',
            zone_format='IPV6',
            ns_group='my_ns_group'
        )
        self.backend.restart_if_needed.assert_called()

    def test_create_zone_no_dns_view(self):
        self.backend.dns_view = None

        self.assertRaisesRegex(
            exceptions.Backend,
            'Unable to create zone. No DNS View found',
            self.backend.create_zone, self.admin_context, self.zone
        )

    def test_delete_zone(self):
        self.backend.restart_if_needed = mock.Mock()

        self.backend.delete_zone(self.admin_context, self.zone)

        self.backend.infoblox.delete_dns_zone.assert_called_with(
            'my_dns_view',
            'example.com'
        )
        self.backend.restart_if_needed.assert_called()

    def test_delete_zone_handle_error(self):
        self.backend.infoblox.delete_dns_zone.side_effect = (
            infoblox_exceptions.InfobloxTimeoutError('error')
        )

        self.assertRaisesRegex(
            exceptions.Backend,
            'Connection to NIOS timed out',
            self.backend.delete_zone, self.admin_context, self.zone
        )

    def test_delete_zone_no_dns_view(self):
        self.backend.dns_view = None

        self.assertRaisesRegex(
            exceptions.Backend,
            'Unable to delete zone. No DNS View found',
            self.backend.delete_zone, self.admin_context, self.zone
        )


class AdvancedInfobloxBackendTestCase(InfobloxBackendTestCase):
    def setUp(self):
        super().setUp()

        self.target['options'].append(
            {'key': 'multi_tenant', 'value': '1'},
        )

        self.backend = impl_infoblox.InfobloxBackend(
            objects.PoolTarget.from_dict(self.target)
        )

    @requests_mock.mock()
    def test_create_zone(self, req_mock):
        zone_name = self.zone['name'][0:-1]
        network_view = f'{self.network_view}.{self.project_id}'
        view_name = f'{self.dns_view}.{self.network_view}.{self.project_id}'

        req_mock.get(
            f'{self.base_address}networkview?*TenantID={self.project_id}'
            '&_return_fields=name',
            json=[{
                '_ref': f'networkview/mock:{network_view}/false',
                'name': f'{network_view}'
            }]
        )
        req_mock.get(
            f'{self.base_address}view?name={view_name}&_return_fields=name',
            json=[{
                '_ref': f'view/mock:{view_name}/false',
                'name': f'{view_name}'
            }]
        )
        req_mock.get(
            f'{self.base_address}zone_auth?fqdn={zone_name}&view={view_name}',
            json=[{
                '_ref': f'zone_auth/mock:{zone_name}/{view_name}',
                'extattrs': {},
                'fqdn': zone_name,
                'grid_primary': [],
                'grid_secondaries': [],
                'ns_group': self.ns_group,
                'view': view_name
            }]
        )
        req_mock.get(
            f'{self.base_address}grid',
            json=[{'_ref': 'grid/mock:Infoblox'}]
        )
        req_mock.post(
            f'{self.base_address}grid/mock%3AInfoblox?'
            '_function=restartservices',
            json=[]
        )

        self.backend.create_zone(self.admin_context, self.zone)

        self.assertEqual(
            req_mock.last_request.json(),
            {
                'mode': 'GROUPED',
                'restart_option': 'RESTART_IF_NEEDED',
                'services': ['DNS']
            }
        )
        self.assertIn('Create Zone', self.stdlog.logger.output)

    @requests_mock.mock()
    def test_delete_zone(self, req_mock):
        zone_name = self.zone['name'][0:-1]
        network_view = f'{self.network_view}.{self.project_id}'
        view_name = f'{self.dns_view}.{self.network_view}.{self.project_id}'

        req_mock.get(
            f'{self.base_address}networkview?*TenantID={self.project_id}'
            '&_return_fields=name',
            json=[{
                '_ref': f'networkview/mock:{network_view}/false',
                'name': f'{network_view}'
            }]
        )
        req_mock.get(
            f'{self.base_address}view?name={view_name}&_return_fields=name',
            json=[{
                '_ref': f'view/mock:{view_name}/false',
                'name': f'{view_name}'
            }]
        )
        req_mock.get(
            f'{self.base_address}zone_auth?fqdn={zone_name}&view={view_name}',
            json=[{
                '_ref': f'zone_auth/mock:{zone_name}/{view_name}',
                'extattrs': {},
                'fqdn': zone_name,
                'grid_primary': [],
                'grid_secondaries': [],
                'ns_group': self.ns_group,
                'view': view_name
            }]
        )
        req_mock.delete(
            f'{self.base_address}zone_auth/mock%3A{zone_name}/{view_name}',
            json=[]
        )
        req_mock.get(
            f'{self.base_address}grid',
            json=[{'_ref': 'grid/mock:Infoblox'}]
        )
        req_mock.post(
            f'{self.base_address}grid/mock%3AInfoblox?'
            '_function=restartservices',
            json=[]
        )

        self.backend.delete_zone(self.admin_context, self.zone)

        self.assertEqual(
            req_mock.last_request.json(),
            {
                'mode': 'GROUPED',
                'restart_option': 'RESTART_IF_NEEDED',
                'services': ['DNS']
            }
        )

        self.assertIn('Delete Zone', self.stdlog.logger.output)



#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from keystoneauth1 import exceptions as kse
from keystoneauth1 import loading as ksa_loading
from oslo_log import log as logging

import designate.conf
from designate import exceptions
from designate.i18n import _


CONF = designate.conf.CONF
LOG = logging.getLogger(__name__)


def verify_project_id(context, project_id):
    """verify that a project_id exists.

    This attempts to verify that a project id exists. If it does not,
    an HTTPBadRequest is emitted.

    """
    session = ksa_loading.load_session_from_conf_options(
        CONF, 'keystone', auth=context.get_auth_plugin()
    )
    adapter = ksa_loading.load_adapter_from_conf_options(
        CONF, 'keystone',
        session=session, min_version=(3, 0), max_version=(3, 'latest')
    )
    try:
        response = adapter.get('/projects/%s' % project_id, raise_exc=False)
    except kse.EndpointNotFound:
        LOG.error(
            'Keystone identity service version 3.0 was not found. This might '
            'be because your endpoint points to the v2.0 versioned endpoint '
            'which is not supported. Please fix this.'
        )
        raise exceptions.KeystoneCommunicationFailure(
            _('KeystoneV3 endpoint not found')
        )
    except kse.ClientException:
        # something is wrong, like there isn't a keystone v3 endpoint,
        # or nova isn't configured for the interface to talk to it;
        # we'll take the pass and default to everything being ok.
        LOG.info('Unable to contact keystone to verify project_id')
        return True

    if response.ok:
        # All is good with this 20x status
        return True
    elif response.status_code == 403:
        # we don't have enough permission to verify this, so default
        # to "it's ok".
        LOG.info(
            'Insufficient permissions for user %(user)s to verify '
            'existence of project_id %(pid)s',
            {
                'user': context.user_id,
                'pid': project_id
            }
        )
        return True
    elif response.status_code == 404:
        # we got access, and we know this project is not there
        raise exceptions.InvalidProject(
            _('%s is not a valid project ID.') % project_id
        )
    else:
        LOG.warning(
            'Unexpected response from keystone trying to '
            'verify project_id %(pid)s - response: %(code)s %(content)s',
            {
                'pid': project_id,
                'code': response.status_code,
                'content': response.content
            }
        )
        return True



# Copyright (c) 2014 Rackspace Hosting
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from designate.objects import base
from designate.objects import fields


@base.DesignateRegistry.register
class Tenant(base.DesignateObject, base.DictObjectMixin):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    fields = {
        'id': fields.AnyField(nullable=True),
        'zone_count': fields.AnyField(nullable=True),
        'zones': fields.AnyField(nullable=True)
    }

    STRING_KEYS = [
        'id'
    ]


@base.DesignateRegistry.register
class TenantList(base.ListObjectMixin, base.DesignateObject):
    LIST_ITEM_TYPE = Tenant

    fields = {
        'objects': fields.ListOfObjectsField('Tenant'),
    }



# Copyright 2013 Hewlett-Packard Development Company, L.P.
#
# Author: Kiall Mac Innes <kiall@hpe.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import pecan

from designate.api.v2.controllers import rest
import designate.conf


CONF = designate.conf.CONF


class LimitsController(rest.RestController):

    @pecan.expose(template='json:', content_type='application/json')
    def get_all(self):
        context = pecan.request.environ['context']

        absolute_limits = self.central_api.get_absolute_limits(context)

        return {
            # Resource Creation Limits
            "max_zones": absolute_limits['zones'],
            "max_zone_recordsets": absolute_limits['zone_recordsets'],
            "max_zone_records": absolute_limits['zone_records'],
            "max_recordset_records": absolute_limits['recordset_records'],

            # Resource Field Value Limits
            "min_ttl": CONF['service:central'].min_ttl,
            "max_zone_name_length":
                CONF['service:central'].max_zone_name_len,
            "max_recordset_name_length":
                CONF['service:central'].max_recordset_name_len,

            # Resource Fetching Limits
            "max_page_limit": CONF['service:api'].max_limit_v2,
        }



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_log import log as logging
from oslo_utils import importutils

import designate.conf
from designate.context import DesignateContext
import webob.dec

profiler = importutils.try_import("osprofiler.profiler")
profiler_initializer = importutils.try_import("osprofiler.initializer")
profiler_opts = importutils.try_import('osprofiler.opts')
profiler_web = importutils.try_import("osprofiler.web")

CONF = designate.conf.CONF
LOG = logging.getLogger(__name__)

if profiler_opts:
    profiler_opts.set_defaults(CONF)


class WsgiMiddleware:

    def __init__(self, application, **kwargs):
        self.application = application

    @classmethod
    def factory(cls, global_conf, **local_conf):
        if profiler_web:
            return profiler_web.WsgiMiddleware.factory(global_conf,
                                                       **local_conf)

        def filter_(app):
            return cls(app, **local_conf)

        return filter_

    @webob.dec.wsgify
    def __call__(self, request):
        return request.get_response(self.application)


def setup_profiler(binary, host):
    if hasattr(CONF, 'profiler') and not CONF.profiler.enabled:
        return

    if (profiler_initializer is None or profiler is None or
            profiler_opts is None):
        LOG.debug('osprofiler is not present')
        return

    profiler_initializer.init_from_conf(
            conf=CONF,
            context=DesignateContext.get_admin_context().to_dict(),
            project="designate",
            service=binary,
            host=host)
    LOG.info("osprofiler is enabled")


def trace_cls(name, **kwargs):
    """Wrap the OSprofiler trace_cls.

    Wrap the OSprofiler trace_cls decorator so that it will not try to
    patch the class unless OSprofiler is present.

    :param name: The name of action. For example, wsgi, rpc, db, ...
    :param kwargs: Any other keyword args used by profiler.trace_cls
    """

    def decorator(cls):
        if profiler and hasattr(CONF, 'profiler') and CONF.profiler.enabled:
            trace_decorator = profiler.trace_cls(name, kwargs)
            return trace_decorator(cls)
        return cls

    return decorator
